
local respawntime = 30

local FriendRespawn = Class(function(self, inst)
	self.inst = inst
	self.timetilrespawn = respawntime
	self.respawning = false
	self.finishedrespawning = false
	self.period = 1
end)

local function GetSpawnPoint(pt)
    local theta = math.random() * 2 * PI
    local radius = 3

	local offset = FindWalkableOffset(pt, theta, radius, 6, true)
	if offset then
		return pt+offset
	end
end

function FriendRespawn:DoUpdate(dt)
	local RespawningPlayer = self.inst.otherpartymember
	local AlivePlayer = self.inst
	local PlayerPos = Point(self.inst.Transform:GetWorldPosition())
		
	self.timetilrespawn = self.timetilrespawn - dt
	--print("Respawning in "..self.timetilrespawn)
	
	if RespawningPlayer and not RespawningPlayer.components.health:IsDead() then
		self:StopRespawning()
		
	elseif self.timetilrespawn <= 0 and not AlivePlayer.components.health:IsDead() and AlivePlayer:IsValid() and self.respawning and not self.finishedrespawning then		
		self.finishedrespawning = true
		SpawnPrefab("collapse_small").Transform:SetPosition(RespawningPlayer.Transform:GetWorldPosition())	
		RespawningPlayer:Hide()	
		
		local spawnpoint = GetSpawnPoint(Vector3(AlivePlayer.Transform:GetWorldPosition()))	
		RespawningPlayer.Physics:Teleport(spawnpoint:Get())		
		
		scheduler:ExecuteInTime(1, function()
			self:StopRespawning()
			RespawningPlayer:Show()
			GetSeasonManager():DoLightningStrike(Vector3(RespawningPlayer.Transform:GetWorldPosition()))
			RespawningPlayer.SoundEmitter:PlaySound("dontstarve/common/resurrectionstone_break")

			if RespawningPlayer.components.hunger then
				RespawningPlayer.components.hunger:SetPercent(2/3)
			end

			if RespawningPlayer.components.health then
				RespawningPlayer.components.health:Respawn(50)
			end
			
			if RespawningPlayer.components.sanity then
				RespawningPlayer.components.sanity:SetPercent(0.5)
			end
			
			if RespawningPlayer == GetPlayer() then
				GetPlayer().HUD.controls.inv:Show()
				GetPlayer().HUD.controls.status:Show()
				GetPlayer().HUD.controls.crafttabs:Show()
			else
				GetPlayer().HUD.controls.inv2:Show()
				GetPlayer().HUD.controls.status2:Show()
				--GetPlayer().HUD.controls.crafttabs:Hide()
			end
			
			if RespawningPlayer.components.revivalsickness then
				RespawningPlayer.components.revivalsickness:BeginSickness()
			end
			
			RespawningPlayer.sg:GoToState("wakeup")
		end)
	end	
end

function FriendRespawn:ResetRespawning()
    self.timetilrespawn = respawntime
end

function FriendRespawn:StartRespawning()
	if not self.respawning and self.inst.otherpartymember then
		if self.inst ~= GetPlayer() then
			self.inst.components.playerleash:StopLeashing()
			TheCamera:SetTarget(self.inst)
			TheCamera:SetDefault()
			TheCamera:Snap()
		else
			self.inst.partymember.components.playerleash:StopLeashing()
		end
		
		self.inst:ListenForEvent("healthdelta", function(inst, data) 
			if data.oldpercent > data.newpercent then
				self:ResetRespawning()
			end
		end, self.inst)
			
		local amulet = nil
		if self.inst.otherpartymember.components.inventory and self.inst.otherpartymember.components.resurrectable and self.inst.otherpartymember.components.resurrectable:CanResurrect() then
			local item = self.inst.otherpartymember.components.inventory:GetEquippedItem(EQUIPSLOTS.BODY)
			if item and item.prefab == "amulet" then
				amulet = true
			end
		end	
			
		if not amulet then	
			self.inst.otherpartymember.components.health:SetInvincible(true)
			if self.task == nil then
				self.task = self.inst:DoPeriodicTask(self.period, function() self:DoUpdate(self.period) end)
			end
			self.respawning = true
		end
	end
end

function FriendRespawn:StopRespawning()
	if self.inst ~= GetPlayer() and self.finishedrespawning then		
		TheCamera:SetTarget(GetPlayer())
		TheCamera:SetDefault()
		TheCamera:Snap()
	end
	
	if self.inst.otherpartymember then
		self.inst.partymember.components.playerleash:StartLeashing()
		self.inst.otherpartymember.components.health:SetInvincible(false)
	end
	
	self.timetilrespawn = respawntime
    self.respawning = false
	self.finishedrespawning = false
	
    if self.task then
        self.task:Cancel()
        self.task = nil
    end
end

function FriendRespawn:IsRespawning()
	return self.respawning
end

function FriendRespawn:CanRespawn(wilson)
	if wilson.otherpartymember and wilson.otherpartymember.components.health and not wilson.otherpartymember.components.health:IsDead() then
		return true
	else
		return false
	end
end

function FriendRespawn:RespawnByAmulet(wilson)
	if wilson.components.inventory and wilson.components.resurrectable and wilson.components.resurrectable:CanResurrect() then
		local item = wilson.components.inventory:GetEquippedItem(EQUIPSLOTS.BODY)
		if item and item.prefab == "amulet" then
			return true
		end
	end
	return false
end

function FriendRespawn:RespawnByDevice(wilson)
	if wilson.components.inventory and wilson.components.resurrectable and wilson.components.resurrectable:CanResurrect() then
		local item = wilson.components.inventory:GetEquippedItem(EQUIPSLOTS.BODY)
		if not (item and item.prefab == "amulet") then
			return true
		end
	end
	return false
end

return FriendRespawn
