require "class"
require "screens/pausescreen"
local easing = require "easing"

local trace = function() end

local CLICK_WALK_TIME = .5

local CameraRight = TheCamera:GetRightVec()
local CameraDown = TheCamera:GetDownVec()

local function UpdateCameraHeadings()
	CameraRight = TheCamera:GetRightVec()
	CameraDown = TheCamera:GetDownVec()
end

local PlayerController = Class(function(self, inst)
    self.inst = inst
    self.enabled = true
    self.follower = false
	
    if self.inst ~= GetPlayer() then
		print(tostring(self.inst).." is the second player")
		self.follower = true
	end
	
    self.inputhandlers = {}
	
	if not self.follower then
	
		self.actionkey = KEY_SPACE
		self.modifier1key = nil
		self.modifier2key = nil
		self.upkey = KEY_W
		self.downkey = KEY_S
		self.leftkey = KEY_A
		self.rightkey = KEY_D
		self.forcekey = KEY_CTRL
		self.attackkey = KEY_Z
		
		
	    table.insert(self.inputhandlers, TheInput:AddKeyUpHandler(KEY_P, function() self:SpawnPlayer2() end))

		
		table.insert(self.inputhandlers, TheInput:AddControlHandler(CONTROL_PRIMARY, function(isPressed) if isPressed then self:OnLeftClick() else self:OnLeftUp() end end))
		table.insert(self.inputhandlers, TheInput:AddControlHandler(CONTROL_SECONDARY, function(isPressed) if isPressed then self:OnRightClick() end end))    
		
		table.insert(self.inputhandlers, TheInput:AddControlHandler(CONTROL_ACTION, function(isPressed) if isPressed then self:OnPressAction() end end)) 
		table.insert(self.inputhandlers, TheInput:AddKeyDownHandler(self.attackkey, function() self:OnPressAttack() end))
		
		table.insert(self.inputhandlers, TheInput:AddControlHandler(CONTROL_ROTATE_LEFT, function(isPressed) if isPressed then self:RotLeft() end end))
		table.insert(self.inputhandlers, TheInput:AddControlHandler(CONTROL_ROTATE_RIGHT, function(isPressed) if isPressed then self:RotRight() end end))
		table.insert(self.inputhandlers, TheInput:AddControlHandler(CONTROL_ZOOM_IN, function(isPressed) if isPressed and not IsHUDPaused() and TheCamera:CanControl() then TheCamera:ZoomIn() end end))
		table.insert(self.inputhandlers, TheInput:AddControlHandler(CONTROL_ZOOM_OUT, function(isPressed) if isPressed and not IsHUDPaused() and TheCamera:CanControl() then TheCamera:ZoomOut() end end))

		table.insert(self.inputhandlers, TheInput:AddKeyDownHandler(KEY_P, function() if TheInput:IsKeyDown(KEY_CTRL) then TheCamera:SetPaused(not TheCamera.paused) end end))
		table.insert(self.inputhandlers, TheInput:AddKeyDownHandler(KEY_H, function() 
			if TheInput:IsKeyDown(KEY_CTRL) then
				if self.inst.HUD.shown then 
					self.inst.HUD:Hide() 
				else 
					self.inst.HUD:Show() 
				end 
			end
		end))
		
		table.insert(self.inputhandlers, TheInput:AddMouseButtonHandler(MOUSEBUTTON_SCROLLUP, true, function() if not IsHUDPaused() and TheCamera:CanControl() then TheCamera:ZoomIn() end end))
		table.insert(self.inputhandlers, TheInput:AddMouseButtonHandler(MOUSEBUTTON_SCROLLDOWN, true, function() if not IsHUDPaused() and TheCamera:CanControl() then TheCamera:ZoomOut() end end))
	
	else
		-- Redefine keys
		self.actionkey = KEY_KP_0
		self.modifier1key = KEY_KP_ENTER
		self.modifier2key = KEY_KP_PLUS
		self.upkey = KEY_UP
		self.downkey = KEY_DOWN
		self.leftkey = KEY_LEFT
		self.rightkey = KEY_RIGHT
		self.forcekey = KEY_KP_ENTER
		self.attackkey = KEY_KP_PERIOD

		table.insert(self.inputhandlers, TheInput:AddKeyDownHandler(self.actionkey, function() self:OnPressAction() end))
		table.insert(self.inputhandlers, TheInput:AddKeyDownHandler(self.attackkey, function() self:OnPressAttack() end))
		table.insert(self.inputhandlers, TheInput:AddKeyUpHandler(self.forcekey, function() GetPlayer().HUD.controls:ClearActiveItemState() end))

		
		
		----DESBLAT CRAFT EDIT
		
		
	    table.insert(self.inputhandlers, TheInput:AddKeyUpHandler(KEY_PAGEUP, function() self:CraftStone() end))
	    table.insert(self.inputhandlers, TheInput:AddKeyUpHandler(KEY_PAGEDOWN, function() self:CraftTrap() end))
        table.insert(self.inputhandlers, TheInput:AddKeyUpHandler(KEY_END, function() self:CraftFirepit() end))
	    table.insert(self.inputhandlers, TheInput:AddKeyUpHandler(KEY_DELETE, function() self:CraftTorch() end))
        table.insert(self.inputhandlers, TheInput:AddKeyUpHandler(KEY_INSERT, function() self:CraftAxe() end))
        table.insert(self.inputhandlers, TheInput:AddKeyUpHandler(KEY_HOME, function() self:CraftPickaxe() end))
		
		
		----DESBLAT CRAFT EDIT END



	end
       
	self.inst:ListenForEvent( "death", function() TheInput:EnableMouseovers() end)
    self.inst:StartUpdatingComponent(self)
    self.draggingonground = false
    self.startdragtestpos = nil
    self.startdragtime = nil
end)









-------DESBLAT EDIT CRAFT


local function SpawnEffect(inst)
    local pt = inst:GetPosition()
    local fx = SpawnPrefab("small_puff")
    fx.Transform:SetPosition(pt.x, pt.y, pt.z)
    fx.Transform:SetScale(0.5,0.5,0.5)
end





function PlayerController:CraftFirepit()
    if self.inst.components.inventory:Has("log", 2) and self.inst.components.inventory:Has("cutgrass", 1) then
    local rabbit = SpawnPrefab("campfire")
    rabbit.Transform:SetPosition(self.inst.Transform:GetWorldPosition())
    self.inst.components.inventory:ConsumeByName("log", 2)
    self.inst.components.inventory:ConsumeByName("cutgrass", 1)
end
end


function PlayerController:CraftTrap()
    if self.inst.components.inventory:Has("cutgrass", 1) and self.inst.components.inventory:Has("twigs", 1) then
    local rabbit = SpawnPrefab("trap")
    rabbit.Transform:SetPosition(self.inst.Transform:GetWorldPosition())
    self.inst.components.inventory:ConsumeByName("cutgrass", 1)
    self.inst.components.inventory:ConsumeByName("twigs", 1)
    rabbit.components.finiteuses:SetMaxUses(5)
    rabbit.components.finiteuses:SetUses(5)
	SpawnEffect(rabbit)
end
end




function PlayerController:CraftAxe()
    if self.inst.components.inventory:Has("flint", 1) and self.inst.components.inventory:Has("twigs", 1) then
    local rabbit = SpawnPrefab("axe")
    rabbit.Transform:SetPosition(self.inst.Transform:GetWorldPosition())
    self.inst.components.inventory:ConsumeByName("flint", 1)
    self.inst.components.inventory:ConsumeByName("twigs", 1)
	SpawnEffect(rabbit)
end
end



function PlayerController:CraftPickaxe()
    if self.inst.components.inventory:Has("flint", 2) and self.inst.components.inventory:Has("twigs", 1) then
    local rabbit = SpawnPrefab("pickaxe")
    rabbit.Transform:SetPosition(self.inst.Transform:GetWorldPosition())
    self.inst.components.inventory:ConsumeByName("flint", 2)
    self.inst.components.inventory:ConsumeByName("twigs", 1)
	SpawnEffect(rabbit)
end
end

function PlayerController:CraftTorch()
    if self.inst.components.inventory:Has("cutgrass", 2) and self.inst.components.inventory:Has("twigs", 1) then
    local rabbit = SpawnPrefab("torch")
    rabbit.Transform:SetPosition(self.inst.Transform:GetWorldPosition())
    self.inst.components.inventory:ConsumeByName("cutgrass", 2)
    self.inst.components.inventory:ConsumeByName("twigs", 1)
	SpawnEffect(rabbit)
end
end


function PlayerController:CraftStone()
    if self.inst.components.inventory:Has("flint", 6) and self.inst.components.inventory:Has("ash", 3) then
    local rabbit = SpawnPrefab("heatrock")
    rabbit.Transform:SetPosition(self.inst.Transform:GetWorldPosition())
    self.inst.components.inventory:ConsumeByName("flint", 6)
    self.inst.components.inventory:ConsumeByName("ash", 3)
	SpawnEffect(rabbit)
end
end



function PlayerController:SpawnPlayer2()
            local abigail = SpawnPrefab("abigail")
            abigail.Transform:SetPosition(self.inst.Transform:GetWorldPosition())
-------			local bishop_2 = SpawnPrefab("bishop_2")
 -----           bishop_2.Transform:SetPosition(self.inst.Transform:GetWorldPosition())
            self.inst.SoundEmitter:PlaySound("dontstarve/common/ghost_spawn")
			self.inst.components.health:DoDelta(-30)

end



---------------------------
---- DESBLAT EDIT CRAFT ___ END
-----------------------------




function PlayerController:RotLeft()
	local rotamount = GetWorld():IsCave() and 22.5 or 45
	if TheCamera:CanControl() then  
		
		if IsHUDPaused() then
			if GetPlayer().HUD:IsMapShowing() then
				TheCamera:SetHeadingTarget(TheCamera:GetHeadingTarget() + rotamount) 
				TheCamera:Snap()
			end
		else
			TheCamera:SetHeadingTarget(TheCamera:GetHeadingTarget() + rotamount) 
			UpdateCameraHeadings() 
		end
	end
end

function PlayerController:RotRight()
	local rotamount = GetWorld():IsCave() and 22.5 or 45
	if TheCamera:CanControl() then  
		
		if IsHUDPaused() then
			if GetPlayer().HUD:IsMapShowing() then
				TheCamera:SetHeadingTarget(TheCamera:GetHeadingTarget() - rotamount) 
				TheCamera:Snap()
			end
		else
			TheCamera:SetHeadingTarget(TheCamera:GetHeadingTarget() - rotamount) 
			UpdateCameraHeadings() 
		end
	end
end

function PlayerController:OnRemoveEntity()
    for k,v in pairs(self.inputhandlers) do
        v:Remove()
    end
end


function PlayerController:GetHoverTextOverride()
	if self.placer_recipe then
		return STRINGS.UI.HUD.BUILD.. " " .. ( STRINGS.NAMES[string.upper(self.placer_recipe.name)] or STRINGS.UI.HUD.HERE )
	end
end

function PlayerController:CancelPlacement()
	if self.placer then
		self.placer:Remove()
		self.placer = nil
	end
	self.placer_recipe = nil
end


function PlayerController:StartBuildPlacementMode(recipe, testfn)
	self.placer_recipe = recipe
	if self.placer then
		self.placer:Remove()
		self.placer = nil
	end
	self.placer = SpawnPrefab(recipe.placer)
	self.placer.components.placer:SetBuilder(self.inst)
	self.placer.components.placer.testfn = testfn
end


function PlayerController:Enable(val)
    self.enabled = val
end

function PlayerController:SmartTargeting()
	local melee_rad = 12
	local ranged_rad = 16
	
	local tool = self.inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
	local has_melee_weapon = tool and tool.components.weapon and not tool.components.weapon.projectile
	local has_ranged_weapon = tool and tool.components.weapon and tool.components.weapon.projectile
	local force_attack = TheInput:IsKeyDown(self.forcekey)
	
	local x,y,z = self.inst.Transform:GetWorldPosition()
	
	local nearby_ents = nil
	if has_ranged_weapon then
		nearby_ents = TheSim:FindEntities(x,y,z, ranged_rad)
	else
		nearby_ents = TheSim:FindEntities(x,y,z, melee_rad)
	end
	
	local possible_targets = {}
	for k,guy in ipairs(nearby_ents) do		
		if guy ~= self.inst and
			guy ~= self.inst.otherpartymember and
		    guy:IsValid() and 
		    not guy:IsInLimbo() and
		    not (guy.sg and guy.sg:HasStateTag("invisible")) and
		    guy.components.health and not guy.components.health:IsDead() and 
		    guy.components.combat and guy.components.combat:CanBeAttacked(self.inst) then
			
			if not force_attack then
				if guy.components.combat.target == GetPlayer().otherpartymember then
					table.insert(possible_targets, {id = k, name = guy, threat = 1, distance = self.inst:GetDistanceSqToInst(guy)})
				elseif guy.components.combat.target == GetPlayer() then
					table.insert(possible_targets, {id = k, name = guy, threat = 2, distance = self.inst:GetDistanceSqToInst(guy)})
				elseif guy:HasTag("monster") or guy:HasTag("killer") then
					table.insert(possible_targets, {id = k, name = guy, threat = 3, distance = self.inst:GetDistanceSqToInst(guy)})
				elseif guy.components.combat.defaultdamage > 0 and ((guy.components.follower and guy.components.follower.leader ~= GetPlayer()) or (guy.components.follower and guy.components.follower.leader ~= GetPlayer().otherpartymember)) then
					table.insert(possible_targets, {id = k, name = guy, threat = 4, distance = self.inst:GetDistanceSqToInst(guy)})
				else
					table.insert(possible_targets, {id = k, name = guy, threat = 99, distance = self.inst:GetDistanceSqToInst(guy)})
				end
				
			else
				if guy.components.combat.target == GetPlayer().otherpartymember then
					table.insert(possible_targets, {id = k, name = guy, threat = 1, distance = self.inst:GetDistanceSqToInst(guy)})
				elseif guy.components.combat.target == GetPlayer() then
					table.insert(possible_targets, {id = k, name = guy, threat = 2, distance = self.inst:GetDistanceSqToInst(guy)})
				elseif guy.components.combat.defaultdamage == 0 then
					table.insert(possible_targets, {id = k, name = guy, threat = 3, distance = self.inst:GetDistanceSqToInst(guy)})	
				elseif guy.components.combat.defaultdamage > 0 and ((guy.components.follower and guy.components.follower.leader == GetPlayer()) or (guy.components.follower and guy.components.follower.leader == GetPlayer().otherpartymember)) then
					table.insert(possible_targets, {id = k, name = guy, threat = 4, distance = self.inst:GetDistanceSqToInst(guy)})	
				else
					table.insert(possible_targets, {id = k, name = guy, threat = 5, distance = self.inst:GetDistanceSqToInst(guy)})
				end
			end		
		end
	end
	
	table.sort(possible_targets, function(a,b) return (a.threat < b.threat) end)
	local threat_threshold = possible_targets[1].threat
	
	for k,v in pairs(possible_targets) do
		if v.threat > threat_threshold then
			table.remove(possible_targets, k)
		end
	end
	
	table.sort(possible_targets, function(a,b) return (a.distance < b.distance) end)
	
	local selected_target = nil
	if possible_targets[1].threat ~= 99 then
		selected_target = possible_targets[1].name
	end
	
	--[[for k,v in pairs(possible_targets) do
		print(k,v.name)
		print(k,v.threat)
		print(k,v.distance)
	end
	print("")]]
	return selected_target
end

function PlayerController:GetAttackTarget()

	local x,y,z = self.inst.Transform:GetWorldPosition()
	local force_attack = TheInput:IsKeyDown(KEY_KP_ENTER)
	local rad = force_attack and 8 or 6 
	local nearby_ents = TheSim:FindEntities(x,y,z, rad)
	local tool = self.inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
	local has_weapon = tool and tool.components.weapon 
	
	for k,guy in ipairs(nearby_ents) do		
		if guy ~= self.inst and
			guy ~= self.inst.otherpartymember and
		    guy:IsValid() and 
		    not guy:IsInLimbo() and
		    not (guy.sg and guy.sg:HasStateTag("invisible")) and
		    guy.components.health and not guy.components.health:IsDead() and 
		    guy.components.combat and guy.components.combat:CanBeAttacked(self.inst) and
		    not (guy.components.follower and guy.components.follower.leader == self.inst) then
				if (guy:HasTag("monster") and has_weapon) or
					guy.components.combat.target == self.inst or
					force_attack then
						return guy
				end	
		end
	end
end


--
function PlayerController:OnPressAttack()
	if self.actionbuttonoverride then
		self.actionbuttonoverride(self.inst)
		return
	end
	
	if self.enabled and not (self.inst.sg:HasStateTag("working") or self.inst.sg:HasStateTag("doing")) then
		--local attack_target = self:GetAttackTarget() 
		local attack_target = self:SmartTargeting()
		local examine_attack_target = TheInput:IsKeyDown(KEY_KP_PLUS)
		--print(attack_target)
 		
		if attack_target and not examine_attack_target then
			if self.inst.components.combat.target ~= attack_target or self.inst.sg:HasStateTag("idle") then
				local action = BufferedAction(self.inst, attack_target, ACTIONS.ATTACK)
				self.inst.components.locomotor:PushAction(action, true)
			end
			return
			
		elseif attack_target and examine_attack_target then
			local action = BufferedAction(self.inst, attack_target, ACTIONS.LOOKAT)
			self.inst.components.locomotor:PushAction(action, true)
		end
	end
end


function PlayerController:OnPressAction()
	if self.actionbuttonoverride then
		self.actionbuttonoverride(self.inst)
		return
	end

	GetPlayer().HUD.controls:ClearActiveItemState()
	local tool = self.inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS)
	local force_on = TheInput:IsKeyDown(self.forcekey)	
	local rad = 0
	
	if self.enabled and not (self.inst.sg:HasStateTag("working") or self.inst.sg:HasStateTag("doing")) then
		-- Bug catching
		if tool and tool.components.tool and tool.components.tool:CanDoAction(ACTIONS.NET) then
			local target = FindEntity(self.inst, 5, 
				function(guy) 
					return  guy.components.health and not guy.components.health:IsDead() and 
							guy.components.workable and
							guy.components.workable.action == ACTIONS.NET
				end)
			if target then
				local action = BufferedAction(self.inst, target, ACTIONS.NET, tool)
				self.inst.components.locomotor:PushAction(action, true)
				return
			end
		end
	 
		-- Projectile catching
		rad = 8
		local projectile = FindEntity(self.inst, rad, function(guy)
			return guy.components.projectile
				   and guy.components.projectile:IsThrown()
				   and self.inst.components.catcher
				   and self.inst.components.catcher:CanCatch()
		end)
		if projectile then
			self.inst.components.locomotor:PushAction(BufferedAction(self.inst, projectile, ACTIONS.CATCH), true)
			return
		end
		
		-- Pickup / activate / look at item
		rad = 6
		local examine_on = nil
		local pickup = nil
		
		if self.modifier2key then
			examine_on = TheInput:IsKeyDown(self.modifier2key)
		end
		
		if examine_on then
			local examine = FindEntity(self.inst, rad, function(guy) return (guy and not guy:HasTag("player")) end)
			if examine then
				local action = BufferedAction(self.inst, examine, ACTIONS.LOOKAT)
				self.inst.components.locomotor:PushAction(action, true)
			end
		else
			pickup = FindEntity(self.inst, rad, function(guy) return (guy.components.inventoryitem and guy.components.inventoryitem.canbepickedup) or
				(tool and tool.components.tool and guy.components.workable and tool.components.tool:CanDoAction(guy.components.workable.action)) or
				(guy.components.pickable and guy.components.pickable:CanBePicked() and guy.components.pickable.caninteractwith) or
				(guy.components.crop and guy.components.crop:IsReadyForHarvest()) or
				(guy.components.harvestable and guy.components.harvestable:CanBeHarvested()) or
				(guy.components.trap and guy.components.trap.issprung) or
				(guy.components.stewer) or
				(guy.components.dryer and guy.components.dryer:IsDone()) or
				(guy.components.container) or
				(guy.components.activatable and guy.components.activatable.inactive) and not
				(guy:HasTag("player"))
			end)
		end
		local has_active_item = self.inst.components.inventory:GetActiveItem() ~= nil

		if pickup and not has_active_item then
			local do_not_activate = false
			local action = nil
			--local activate_on = TheInput:IsKeyDown(self.modifier1key)
			
			if (tool and tool.components.tool and pickup.components.workable and tool.components.tool:CanDoAction(pickup.components.workable.action)) then
				action = pickup.components.workable.action
			elseif pickup.components.trap and pickup.components.trap.issprung then
				action = ACTIONS.CHECKTRAP
			elseif pickup.components.activatable and pickup.components.activatable.inactive then
				action = ACTIONS.ACTIVATE
			elseif pickup.components.inventoryitem and pickup.components.inventoryitem.canbepickedup and not (pickup:HasTag("teleportato_part") and self.follower) then 
				action = ACTIONS.PICKUP 
			elseif pickup.components.pickable and pickup.components.pickable:CanBePicked() then 
				action = ACTIONS.PICK 		
			elseif pickup.components.harvestable and pickup.components.harvestable:CanBeHarvested() then
				action = ACTIONS.HARVEST
			elseif pickup.components.crop and pickup.components.crop.matured then
				action = ACTIONS.HARVEST				
			elseif pickup.components.dryer and pickup.components.dryer.product then
				action = ACTIONS.HARVEST
			elseif pickup.components.stewer and pickup.components.stewer.product then
				action = ACTIONS.HARVEST
			elseif pickup.components.stewer and pickup.components.stewer:CanCook() and not pickup.components.stewer.cooking and pickup.components.container and pickup.components.container:IsOpen() then
				--pickup.components.stewer:StartCooking()	
				pickup.components.container.widgetbuttoninfo.fn(pickup, GetPlayer())
				pickup.components.container:Close()	
				do_not_activate = true
			elseif pickup.components.activatable and pickup.components.activatable.inactive then
				action = ACTIONS.ACTIVATE			
			end
			
			local activate_on = nil
			if self.modifier1key and not do_not_activate then
				activate_on = TheInput:IsKeyDown(self.modifier1key)
			end
			
			if activate_on and not pickup.components.container then
				local actionlist = self.inst.components.playeractionpicker:GetSceneActions(pickup, true)		
				self.inst.components.locomotor:PushAction(actionlist[1], true)	
			
			elseif activate_on and pickup.components.container then
				local distsq = self.inst:GetDistanceSqToInst(pickup)
				if distsq <= 3*3 and not pickup.components.container:IsOpen() then
					pickup.components.container:Open()
				elseif distsq <= 3*3 and pickup.components.container:IsOpen() then
					pickup.components.container:Close()
				end
					
			elseif action then
				self.inst.components.locomotor:PushAction(BufferedAction(self.inst, pickup, action, tool), true)			
			end
			
			return
		end
	end	
end

function PlayerController:OnUpdate(dt)
    if not self.enabled then 
		TheInput:EnableMouseovers()
		return 
    end  
	
	local active_item = self.inst.components.inventory:GetActiveItem()	
	
	local action = self.inst.components.playeractionpicker:GetLeftMouseAction()	
	local action_r = self.inst.components.playeractionpicker:GetRightMouseAction()	
	
	local terraform_l =	(action and action.action == ACTIONS.TERRAFORM and action.pos and action.invobject)
	local terraform_r =	(action_r and action_r.action == ACTIONS.TERRAFORM and action_r.pos and action_r.invobject)
	local terraform = terraform_l or terraform_r
	
	if terraform then
		if not self.terraformplacer then
			local act = terraform_l and action or action_r
			self.terraformplacer = SpawnPrefab("gridplacer")
			self.terraformplacer.components.placer:SetBuilder(self.inst)
			self.terraformplacer.components.placer.testfn = function(pt) 
				return act.invobject.components.terraformer:CanTerraformPoint(pt) 
			end 
			
		end
	else
		if self.terraformplacer then
			self.terraformplacer:Remove()
			self.terraformplacer = nil
		end
	end
	
	local show_deploy_placer = active_item and active_item.components.deployable and ((action and action.action == ACTIONS.DEPLOY) or (action_r and action_r.action == ACTIONS.DEPLOY))
	
	if show_deploy_placer and not self.follower then -- MOD
		local placer_name = active_item.components.deployable.placer or ((active_item.prefab or "") .. "_placer")
		if self.deployplacer and self.deployplacer.prefab ~= placer_name then
			self.deployplacer:Remove()
			self.deployplacer = nil
		end
		
		if not self.deployplacer then
			self.deployplacer = SpawnPrefab(placer_name)
			if self.deployplacer then
				self.deployplacer.components.placer:SetBuilder(self.inst)
				self.deployplacer.components.placer.testfn = function(pt) 
						local action = self.inst.components.playeractionpicker:GetLeftMouseAction()
						local action_r = self.inst.components.playeractionpicker:GetRightMouseAction()
						return (action and action.action == ACTIONS.DEPLOY and action.invobject and action.invobject.components.deployable and action.invobject.components.deployable:CanDeploy(pt)) or
							   (action_r and action_r.action == ACTIONS.DEPLOY and action_r.invobject and action_r.invobject.components.deployable and action_r.invobject.components.deployable:CanDeploy(pt))
						
					end
				self.deployplacer.components.placer:OnUpdate(0)  --so that our position is accurate on the first frame
			end
		end
	else
		if self.deployplacer then
			self.deployplacer:Remove()
			self.deployplacer = nil
		end
	end
	    
    --if self.placer_recipe then return end

    -- if self.startdragtestpos and not self.draggingonground then
    --     local pt = TheInput:GetWorldPosition()
    --     if distsq(pt, self.startdragtestpos) > 1.5*1.5 then
    --         self.draggingonground = true
    --     end
    -- end
    
    if self.startdragtime and not self.draggingonground then
        local now = GetTime()
        if now - self.startdragtime > CLICK_WALK_TIME then
            self.draggingonground = true
        end
    end

	if not self.inst.sg:HasStateTag("busy") then
		if self.startdragtime then
			local pt = TheInput:GetWorldPosition()
			local dst = distsq(pt, Vector3(self.inst.Transform:GetWorldPosition()))
			if dst > 1 then
				local angle = self.inst:GetAngleToPoint(pt)
				--self.inst.components.locomotor:GoToPoint(pt, nil, true)
				self.inst.components.locomotor:RunInDirection(angle)
				self.inst:ClearBufferedAction()
				TheInput:DisableMouseovers()            
			end
			self.directwalking = false
		else
	        
			--WASD walking!
			local xwalk = 0
			local ywalk = 0
			if TheInput:IsKeyDown(self.upkey) then
				ywalk = ywalk + 1
			end

			if TheInput:IsKeyDown(self.downkey) then
				ywalk = ywalk - 1
			end

			if TheInput:IsKeyDown(self.leftkey) then
				xwalk = xwalk - 1
			end
	        
			if TheInput:IsKeyDown(self.rightkey) then
				xwalk = xwalk + 1
			end

			if not 	TheInput:IsKeyDown(self.upkey) and not --We only want to update headings if no keys are pressed.
					TheInput:IsKeyDown(self.leftkey) and not
					TheInput:IsKeyDown(self.downkey) and not
					TheInput:IsKeyDown(self.rightkey) then

					
					CameraRight = TheCamera:GetRightVec()
					CameraDown = TheCamera:GetDownVec()
			end
	        
			if xwalk ~= 0 or ywalk ~= 0 then
				--self.inst.components.inventory:DropActiveItem()

				local dir = CameraRight * xwalk - CameraDown * ywalk
				dir = dir:GetNormalized()
				--local pt = Vector3(self.inst.Transform:GetWorldPosition()) + dir
				--self.inst.components.locomotor:GoToPoint(pt, nil, true)
				local ang = -math.atan2(dir.z, dir.x)/DEGREES
				self.inst.components.locomotor:WalkInDirection(ang, true)
				self.directwalking = true
				self.inst.components.locomotor:SetBufferedAction(nil)
				self.inst:ClearBufferedAction()
				
				if not self.inst.sg:HasStateTag("attack") then
					self.inst.components.combat:SetTarget(nil)
				end
			else
				if self.directwalking then
					self.inst.components.locomotor:Stop()
					self.directwalking = false
				end
			end
		end
    end
    
end



function PlayerController:OnLeftUp()
	
	Print(VERBOSITY.DEBUG, "OnLeftUp")

	local walk_key_down = TheInput:IsKeyDown(self.upkey) or TheInput:IsKeyDown(self.leftkey) or TheInput:IsKeyDown(self.downkey) or TheInput:IsKeyDown(self.rightkey)

    if not self.enabled then return end    
    
    if not self.ignore_left_up and not TheInput:GetHUDEntityUnderMouse() then 
        if self.inst.components.inventory:GetActiveItem() then
            self:DoAction()
        else
			self.directwalking = false
   			if self.draggingonground then
				Print(VERBOSITY.DEBUG, "    stopping")
				if not walk_key_down then
					self.inst.components.locomotor:Stop()
				end
			elseif self.startdragtime then
   				Print(VERBOSITY.DEBUG, "    not dragging")
				local pt = TheInput:GetWorldPosition()
				local dst = distsq(pt, Vector3(self.inst.Transform:GetWorldPosition()))
				if dst > 1 then
					Print(VERBOSITY.DEBUG, "    ending with pathfind")
					--local angle = self.inst:GetAngleToPoint(pt)
					self.inst.components.locomotor:GoToPoint(pt, nil, true)
					--self.inst.components.locomotor:RunInDirection(angle)
					self.inst:ClearBufferedAction()
					--TheInput:DisableMouseovers()
				else
					Print(VERBOSITY.DEBUG, "    stopping")
					self.inst.components.locomotor:Stop()
				end
			 --    local action = BufferedAction(self.inst, target, ACTIONS.WALKTO)
				-- self.inst:ClearBufferedAction()
				-- self.inst.components.locomotor:PushAction(action, true)

			end
        end
    end

    self.ignore_left_up = false
    self.draggingonground = false
    self.startdragtestpos = nil
    self.startdragtime = nil
    TheInput:EnableMouseovers()
end



function PlayerController:DoAction(buffaction)
	Print(VERBOSITY.DEBUG, "PlayerController:DoAction")
    if buffaction then
    
        if self.inst.bufferedaction then
            if self.inst.bufferedaction.action == buffaction.action and self.inst.bufferedaction.target == buffaction.target then
                return;
            end
        end
        
        if buffaction.target and buffaction.target.components.highlight then
            buffaction.target.components.highlight:Flash(.2, .125, .1)
        end
		
		Print(VERBOSITY.DEBUG, "Performing Action: ["..buffaction.action.id.."]")
        if  buffaction.invobject and 
            buffaction.invobject.components.equippable and 
            buffaction.invobject.components.equippable.equipslot == EQUIPSLOTS.HANDS and 
            (buffaction.action ~= ACTIONS.DROP and buffaction.action ~= ACTIONS.STORE) then
            
                if not buffaction.invobject.components.equippable.isequipped then 
                    self.inst.components.inventory:Equip(buffaction.invobject)
					print("action")
                end
                
                if self.inst.components.inventory:GetActiveItem() == buffaction.invobject then
                    self.inst.components.inventory:SetActiveItem(nil)
                end
        end
        
        self.inst.components.locomotor:PushAction(buffaction, true)
    end    

end


function PlayerController:OnLeftClick()
    
    self.startdragtime = nil

    if not self.enabled then return end
    
    if TheInput:GetHUDEntityUnderMouse() then 
		self:CancelPlacement()
		return 
    end

	if self.placer_recipe then
		--do the placement
		if self.placer.components.placer.can_build then
			self.inst.components.builder:MakeRecipe(self.placer_recipe, TheInput:GetWorldPosition())
			self:CancelPlacement()
		end
		return
	end
    
    if self.inst.components.inventory:GetActiveItem() then
        self.ignore_left_up = true
    end
    
    self.inst.components.combat.target = nil
    
    if self.inst.inbed then
        self.inst.inbed.components.bed:StopSleeping()
        return
    end
    
    local action = self.inst.components.playeractionpicker:GetLeftMouseAction()
    if action then
	    self:DoAction( action )
	else
	    local clicked = TheInput:GetWorldEntityUnderMouse()
	    if not clicked then
	        self.startdragtime = GetTime()
	    end
    end
    
end


function PlayerController:OnRightClick()

    self.startdragtime = nil

	if self.placer_recipe then 
		self:CancelPlacement()
	end

    if not self.enabled then return end
    
    if TheInput:GetHUDEntityUnderMouse() then return end

    if not self.inst.components.playeractionpicker:GetRightMouseAction() then
        self.inst.components.inventory:ReturnActiveItem()
    end
    
    if self.inst.inbed then
        self.inst.inbed.components.bed:StopSleeping()
        return
    end
    
    local action = self.inst.components.playeractionpicker:GetRightMouseAction()
    if action then
		self:DoAction(action )
	end
		
    
end

function PlayerController:ShakeCamera(inst, shakeType, duration, speed, maxShake, maxDist)
    local distSq = self.inst:GetDistanceSqToInst(inst)
    local t = math.max(0, math.min(1, distSq / (maxDist*maxDist) ) )
    local scale = easing.outQuad(t, maxShake, -maxShake, 1)
    if scale > 0 then
        TheCamera:Shake(shakeType, duration, speed, scale)
    end
end

return PlayerController
