
local PlayerLeash = Class(function(self, inst)
	self.inst = inst
	self.timetilleash = 0.5
	self.period = 0.1
	self.leashed = false
	self.returning = false
	self.normrunspeed = self.inst.components.locomotor.runspeed
	self.increasedrunspeed = self.inst.components.locomotor.runspeed*1.5
end)

local function GetSpawnPoint(pt)
    local theta = math.random() * 2 * PI
    local radius = 6

	local offset = FindWalkableOffset(pt, theta, radius, 6, true)
	if offset then
		return pt+offset
	end
end

function PlayerLeash:DoUpdate(dt)
	
	if GetPlayer().components.health:IsDead() and self.leashed then
		self.inst.components.playerleash:StopLeashing()
		GetPlayer().components.health:IsInvincible()
		TheCamera:SetTarget(self.inst)
		TheCamera:SetDefault()
		TheCamera:Snap()
	elseif self.inst ~= GetPlayer and self.inst.components.health:IsDead() and self.leashed then
		self.inst.components.playerleash:StopLeashing()
	end
	
	local LeashDist = 22
	local LeashRemoveDist = 17
	local TeleportDist = 30
	
	local Player1Pos = Point(GetPlayer().Transform:GetWorldPosition())
	local Player2Pos = Point(self.inst.Transform:GetWorldPosition())	
	local DistSq = distsq(Player2Pos, Player1Pos)
	
	self.timetilleash = self.timetilleash - dt
	
	if DistSq > TeleportDist*TeleportDist and self.leashed then		
		local spawnpoint = GetSpawnPoint(Vector3(GetPlayer().Transform:GetWorldPosition()))	
		self.inst.Physics:Teleport(spawnpoint:Get())
		SpawnPrefab("collapse_small").Transform:SetPosition(self.inst.Transform:GetWorldPosition())	
		self.inst.components.locomotor:Stop()
		
	elseif DistSq > LeashDist*LeashDist and self.leashed then				
		if self.timetilleash <= 0 then
			--print("run")
			self.timetilleash = 0.5
			self.returning = true
			self.inst.components.locomotor.runspeed = self.increasedrunspeed
			self.inst.components.playercontroller.enabled = false
			
			local theta = math.random() * 2 * PI
			local radius = 2
			local offset = FindWalkableOffset(Player1Pos, theta, radius, 8, true)
			if offset then
				Player1Pos = Player1Pos + offset
			end
			
			self.inst.components.locomotor:GoToPoint(Player1Pos, nil, true)
		end
	
	elseif DistSq <= LeashRemoveDist*LeashRemoveDist and GetPlayer().sg.state ~= "jumpin" and self.leashed then
		self.inst.components.playercontroller.enabled = true
		self.timetilleash = 0.5
		self.returning = false
		self.inst.components.locomotor.runspeed = self.normrunspeed
	end	
end

function PlayerLeash:StartLeashing()
    self.leashed = true
    if self.task == nil then
        self.task = self.inst:DoPeriodicTask(self.period, function() self:DoUpdate(self.period) end)
    end
end

function PlayerLeash:StopLeashing()
    self.leashed = false
    if self.task then
        self.task:Cancel()
        self.task = nil
    end
	if self.returning then
		self.inst.components.locomotor.runspeed = self.normrunspeed
		self.inst.components.locomotor:Stop()
		self.returning = false
	end
end

return PlayerLeash
