require("widgets/statusdisplays")
require("widgets/respawntimer")
require("widgets/revived")
require("class")
require("tilebg")
require("tabs")
require("widgets/inventoryslot")

local firstrun = 0
local SaveGameIndex = SaveIndex()

local PlayerMonitor = Class(function(self, inst)
    self.inst = inst
	self.inst:StartUpdatingComponent(self)
	self.maxwellspeech = nil
end)

function PlayerMonitor:OnSave()
	if self.player2prefab then
		return {player2prefab = self.player2prefab}
	else
		return
	end
end

function PlayerMonitor:OnLoad(data)
	if data then
		self.player2prefab = data.player2prefab
	end
end

function PlayerMonitor:SetupPartyMember(player2)
	local player1 = GetPlayer()
	
	-- Create party
	player1.partyleader = player1
	player1.partymember = player2
	player1.otherpartymember = player2
	
	player2.partyleader = player1
	player2.partymember = player2
	player2.otherpartymember = player1
	
	-- Set up second player HUD
	player1.HUD.controls.inv2 = player1.HUD.controls.bottom_root:AddChild(Inv(player2)) 
	player1.HUD.controls.status2 = player1.HUD.controls.toppanel:AddChild(Status(player2))
	player1.HUD.controls.status2:SetPosition(235,0,0)
	
	player1.HUD.controls.respawntimer = player1.HUD.controls.toppanel:AddChild(RespawnTimer(player1))
	player1.HUD.controls.respawntimer:SetPosition(-235,0,0)
	player1.HUD.controls.respawntimer2 = player1.HUD.controls.toppanel:AddChild(RespawnTimer(player2))
	player1.HUD.controls.respawntimer2:SetPosition(235,0,0)
	
	player1.HUD.controls.revived = player1.HUD.controls.toppanel:AddChild(Revived(player1))
	player1.HUD.controls.revived:SetPosition(-385,0,0)
	player1.HUD.controls.revived2 = player1.HUD.controls.toppanel:AddChild(Revived(player2))
	player1.HUD.controls.revived2:SetPosition(385,0,0)
	
	-- Have to refreash the crafting tabs if one player is Wickerbottom
	if player2.prefab == "wickerbottom" then
		--print("wickerbottom!")
		
		local owner = player1
		local tab_bg = 
			{
				normal = "tab_normal.tex",
				selected = "tab_selected.tex",
				highlight = "tab_highlight.tex",
				overlay = "tab_researchable.tex",
			}
		
		player1.HUD.controls.crafttabs:SetPosition(0,0,0)
		
		player1.HUD.controls.crafttabs.crafting = player1.HUD.controls.crafttabs:AddChild(Crafting(player1.HUD.controls.crafttabs, player1.HUD.controls.crafttabs.owner))
		player1.HUD.controls.crafttabs.crafting:Hide()
		player1.HUD.controls.crafttabs.bg = player1.HUD.controls.crafttabs:AddChild(Image("images/hud.xml", "craft_bg.tex"))
		
		player1.HUD.controls.crafttabs.tabs = player1.HUD.controls.crafttabs:AddChild(TabGroup())
		player1.HUD.controls.crafttabs.tabs:SetPosition(-16,0,0)

		player1.HUD.controls.crafttabs.tabs.onopen = function() player1.HUD.controls.crafttabs.owner.SoundEmitter:PlaySound("dontstarve/HUD/craft_open") end
		player1.HUD.controls.crafttabs.tabs.onchange = function() player1.HUD.controls.crafttabs.owner.SoundEmitter:PlaySound("dontstarve/HUD/craft_open") end
		player1.HUD.controls.crafttabs.tabs.onclose = function() player1.HUD.controls.crafttabs.owner.SoundEmitter:PlaySound("dontstarve/HUD/craft_close") end
		player1.HUD.controls.crafttabs.tabs.onhighlight = function() player1.HUD.controls.crafttabs.owner.SoundEmitter:PlaySound("dontstarve/HUD/recipe_ready") return .2 end
		player1.HUD.controls.crafttabs.tabs.onoverlay = function() player1.HUD.controls.crafttabs.owner.SoundEmitter:PlaySound("dontstarve/HUD/research_available") return .2 end
		
		local tabnames = {}
		for k,v in pairs(RECIPETABS) do
			table.insert(tabnames, v)
		end

		for k,v in pairs(owner.components.builder.custom_tabs) do
			table.insert(tabnames, v)
		end

		table.sort(tabnames, function(a,b) return a.sort < b.sort end)
		
		player1.HUD.controls.crafttabs.tabs.spacing = CRAFTING_CONSTANTS.TABBAR_HEIGHT/#tabnames
		
		player1.HUD.controls.crafttabs.tabbyfilter = {}
		for k,v in ipairs(tabnames) do
			local tab = player1.HUD.controls.crafttabs.tabs:AddTab(v.str, resolvefilepath("images/hud.xml"), v.icon_atlas or resolvefilepath("images/hud.xml"), v.icon, tab_bg.normal, tab_bg.selected, tab_bg.highlight, tab_bg.overlay,
				function() --select fn
					player1.HUD.controls.crafttabs.crafting:SetFilter( 
						function(recipe)
							local rec = GetRecipe(recipe)
							
							return rec and rec.tab == v
						end)
					player1.HUD.controls.crafttabs.crafting:Open() 
					
				end, 
				function() --deselect fn
					player1.HUD.controls.crafttabs.crafting:Close()
				end)
			player1.HUD.controls.crafttabs.tabbyfilter[v] = tab
			
		end
		
		player1.HUD.controls.crafttabs.needtoupdate = true
		player1.HUD.controls.crafttabs:DoUpdateRecipes()
	end
	
	-- HUD effects
	player2:ListenForEvent("startfiredamage", function(inst, data) player1.HUD.controls.fireover:TurnOn() end, player2)
    player2:ListenForEvent("stopfiredamage", function(inst, data) player1.HUD.controls.fireover:TurnOff() end, player2)
	player2:ListenForEvent("temperaturedelta", function(inst, data) player1.HUD.controls.iceover:OnIceChange() end, player2)
	
	player2:ListenForEvent("badaura", function(inst, data) return player1.HUD.bloodover:Flash() end, player2)
	player2:ListenForEvent("attacked", function(inst, data) return player1.HUD.bloodover:Flash() end, player2)
	player2:ListenForEvent("startstarving", function(inst, data) player1.HUD.bloodover:TurnOn() end, player2)
	player2:ListenForEvent("stopstarving", function(inst, data) player1.HUD.bloodover:TurnOff() end, player2)
	player2:ListenForEvent("gosane", function(inst, data) player1.HUD:GoSane() end, player2)
	player2:ListenForEvent("goinsane", function(inst, data) player1.HUD:GoInsane() end, player2)
	
	player1.HUD.controls.player2 = player2
	
	--[[player1.HUD.controls.right_root = player1.HUD.controls:AddChild(Widget("side"))
    player1.HUD.controls.right_root:SetScaleMode(SCALEMODE_PROPORTIONAL)
    player1.HUD.controls.right_root:SetHAnchor(ANCHOR_RIGHT)
    player1.HUD.controls.right_root:SetVAnchor(ANCHOR_MIDDLE)
    player1.HUD.controls.right_root:SetMaxPropUpscale(1.25)
	
	player1.HUD.controls.crafttabs2 = player1.HUD.controls.right_root:AddChild(CraftTabs(player2))
    local scale = .75
    player1.HUD.controls.crafttabs2:SetScale(scale,scale,scale)
	player1.HUD.controls.crafttabs2:SetPosition(-135,0,0)]]
end

function PlayerMonitor:SecondPlayerCharacterSelect()
	local player1 = GetPlayer()
	local player2 = nil
	local profile = player1.profile
	local defaultcharacter = nil
		
	if player1.prefab ~= "wilson" then
		defaultcharacter = "wilson"
	else
		defaultcharacter = "willow"
	end
	
	if player1 and profile then 
		
		SetHUDPause(true)
		
		local Screen = nil
		Screen = CharacterSelectScreen(profile, 
			function(Player2Character) 
				TheFrontEnd:PopScreen(Screen)
				
				if Player2Character then	
					TheSim:LoadPrefabs({Player2Character})
					if player1.components.playerprefabsave then
						player1.components.playerprefabsave:PlayerPrefab(Player2Character)
					end
					
					--[[player2 = SpawnPrefab(Player2Character)						
					player2.Transform:SetPosition(player1.Transform:GetWorldPosition())
					player2:SetStateGraph("SGwilson_player2")
					self:SetupPartyMember(player2)	]]						
				end	

				SetHUDPause(false)				
		end, true, defaultcharacter)	
		
		-- Redefine character list
		local characterlist = JoinArrays(CHARACTERLIST, MODCHARACTERLIST)
		
		for k,v in pairs(characterlist) do
			if v == player1.prefab or v == "woodie" then
				table.remove(characterlist, k)
			end
		end
		
		-- Set and refresh character list
		Screen.characters = characterlist
		Screen:SetOffset(0)
		
		TheFrontEnd:PushScreen(Screen)
	
		
	end	
end

local function GetSpawnPoint(pt)
    local theta = math.random() * 2 * PI
    local radius = 3

	local offset = FindWalkableOffset(pt, theta, radius, 6, true)
	if offset then
		return pt+offset
	end
end

function PlayerMonitor:FriendSpawn()
	local player1 = GetPlayer()
	
	-- Spawn the second player in
	local player2 = SpawnPrefab(player1.components.playerprefabsave:SavedPrefab())	
	
	-- New stategraph
	player2:SetStateGraph("SGwilson_player2")
	
	-- Starting inventory
	if player2 and player2.components.inventory and player2.components.inventory.starting_inventory and not GetWorld():IsCave() then
		for k,v in pairs(player2.components.inventory.starting_inventory) do
			local item = SpawnPrefab(v)
			if item then
				player2.components.inventory:GiveItem(item)
			end
		end
	end
	
	-- Arrival
	player2:Hide()	
	local spawnpoint = GetSpawnPoint(Vector3(player1.Transform:GetWorldPosition()))	
	player2.Physics:Teleport(spawnpoint:Get())		
	
	scheduler:ExecuteInTime(1, function()
		player2:Show()
		GetSeasonManager():DoLightningStrike(Vector3(player2.Transform:GetWorldPosition()))
		player2.SoundEmitter:PlaySound("dontstarve/common/resurrectionstone_break")
		player2.sg:GoToState("wakeup")
		self:SetupPartyMember(player2)
	end)
end

function PlayerMonitor:SpawnPartyMember()
	
	-- Spawn player
	local player1 = GetPlayer()
	local player2 = nil
	
	if not (player1.components.playerprefabsave and player1.components.playerprefabsave:SavedPrefab()) then
		self:SecondPlayerCharacterSelect()
					
	elseif not (self.maxwellspeech and self.maxwellspeech.components.maxwelltalker:IsTalking()) then
		self:FriendSpawn()
	end
	
	-- Give starting items
	if player2 and player2.components.inventory and player2.components.inventory.starting_inventory then
		for k,v in pairs(player2.components.inventory.starting_inventory) do
			local item = SpawnPrefab(v)
			if item then
				player2.components.inventory:GiveItem(item)
			end
		end
	end
end

function PlayerMonitor:PartyMemberCheck()
	local player1 = GetPlayer()
	local player2 = nil
	
	for k,v in pairs(Ents) do
		if v:HasTag("player") and v ~= GetPlayer() then 
			player2 = v
		end
	end
	
	if player2 then
		self:SetupPartyMember(player2)
	else
		self:SpawnPartyMember()
	end
end

function PlayerMonitor:OnUpdate(dt)
		
	local player1 = GetPlayer()
	local player2 = player1.otherpartymember
	
	if not player2 then
		self:PartyMemberCheck()
	end
	
	if player1 and player2 then
		assert(player1.prefab ~= "woodie", "\n\n WARNING: Woodie is not supported by the 'Two Player (Local) Co-op' mod at this time.\n Please start a new game and select a different character.\n")
		assert(player2.prefab ~= "woodie", "\n\n WARNING: Woodie is not supported by the 'Two Player (Local) Co-op' mod at this time.\n Please start a new game and select a different character.\n")
	end
	
	-- Woodie
	--[[if player2.prefab == "woodie" and player2.beaver then
		--player2.components.beaverness.is_beaver = true
		GetPlayer().HUD.controls.inv2:Hide()
		GetPlayer().HUD.controls.status2:Hide()
	elseif player2.prefab == "woodie" and not player2.beaver then
		player2.components.beaverness.is_beaver = false
		GetPlayer().HUD.controls.inv2:Show()
		GetPlayer().HUD.controls.status2:Show()
	end]]
	
	-- Respawn a dead player if their friend is still alive
	if player1 and player1.components.health and player1.components.health:IsDead() and player2.components.health and not player2.components.health:IsDead() and player2.components.friendrespawn and not player2.components.friendrespawn:IsRespawning() then
		player2.components.friendrespawn:StartRespawning()
		
	elseif player2 and player2.components.health and player2.components.health:IsDead() and player1.components.health and not player1.components.health:IsDead() and player1.components.friendrespawn and not player1.components.friendrespawn:IsRespawning() then
		player1.components.friendrespawn:StartRespawning()

	elseif player1 and player1.components.health and not player1.components.health:IsDead()then
		if player2 and player2.components.friendrespawn and player2.components.friendrespawn.respawning then
			player2.components.friendrespawn:StopRespawning()
		end

	elseif player2 and player2.components.health and not player2.components.health:IsDead() then
		if player1 and player1.components.friendrespawn and player1.components.friendrespawn.respawning then
			player1.components.friendrespawn:StopRespawning()
		end
	end	
	
	if player1 and player1.components.health and player1.components.health:IsDead() then
		player1.HUD.controls.inv:Hide()
		player1.HUD.controls.status:Hide()
		player1.HUD.controls.crafttabs:Hide()
		player1.HUD.controls.revived:Hide()
		
	elseif player1 then
		player1.HUD.controls.inv:Show()
		player1.HUD.controls.status:Show()
		player1.HUD.controls.crafttabs:Show()
	end
	
	if player2 and player2.components.health and player2.components.health:IsDead() and player1 then
		player1.HUD.controls.inv2:Hide()
		player1.HUD.controls.status2:Hide()
		player1.HUD.controls.revived2:Hide()
		--player1.HUD.controls.crafttabs:Hide()
		--print("hide")
		
	elseif player2 and player1 then	
		player1.HUD.controls.inv2:Show()
		player1.HUD.controls.status2:Show()	
		--player1.HUD.controls.crafttabs2:Show()
		--print("show")
	end
end

return PlayerMonitor


