
local RevivalSickness = Class(function(self, inst)
	self.inst = inst
	self.sickness = 0.7
	self.sicknesstime = 0
	self.sicknessmaxtime = 16*60
	self.period = 1
	self.task = nil
	self.savedhealth = nil
	self.savedhunger = nil
	self.savedsanity = nil
	self.saveddamage = nil
end)

function RevivalSickness:RefreshStats()
	if self.inst.components.health then
		local percent = self.inst.components.health:GetPercent()
		self.inst.components.health.maxhealth = self.inst.components.health.maxhealth*self.sickness
		self.inst.components.health.currenthealth = self.inst.components.health.maxhealth*percent
		self.inst.components.health:DoDelta(0) -- Updates HUD
	end
	
	if self.inst.components.hunger then
		local percent = self.inst.components.hunger:GetPercent()
		self.inst.components.hunger.max = self.inst.components.hunger.max*self.sickness
		self.inst.components.hunger.current = self.inst.components.hunger.max*percent
		self.inst.components.hunger:DoDelta(0) -- Updates HUD
	end
	
	if self.inst.components.sanity then
		local percent = self.inst.components.sanity:GetPercent()
		self.inst.components.sanity.max = self.inst.components.sanity.max*self.sickness
		self.inst.components.sanity.current = self.inst.components.sanity.max*percent
		self.inst.components.sanity:DoDelta(0) -- Updates HUD
	end
	
	if self.inst.components.combat then
		if self.inst.components.combat.damagemultiplier then
			self.inst.components.combat.damagemultiplier = self.inst.components.combat.damagemultiplier*self.sickness
		else
			self.inst.components.combat.damagemultiplier = self.sickness
		end
	end
	
	self.inst:DoTaskInTime(0.2, function()
		if GetPlayer() ~= self.inst then
			GetPlayer().HUD.controls.revived2:Show()
		else
			GetPlayer().HUD.controls.revived:Show()
		end
	end)
end

function RevivalSickness:DoUpdate(dt)
	self.sicknesstime = self.sicknesstime - dt
	--print(self.sicknesstime)
	
	-- Sickness wears off
	if self.sicknesstime <= 0 and self.task then
		if self.inst.components.health then
			local percent = self.inst.components.health:GetPercent()
			self.inst.components.health.maxhealth = self.savedhealth
			self.inst.components.health.currenthealth = self.inst.components.health.maxhealth*percent
			self.inst.components.health:DoDelta(0) -- Updates HUD
		end
		
		if self.inst.components.hunger then
			local percent = self.inst.components.hunger:GetPercent()
			self.inst.components.hunger.max = self.savedhunger
			self.inst.components.hunger.current = self.inst.components.hunger.max*percent
			self.inst.components.hunger:DoDelta(0) -- Updates HUD
		end
		
		if self.inst.components.sanity then
			local percent = self.inst.components.sanity:GetPercent()
			self.inst.components.sanity.max = self.savedsanity
			self.inst.components.sanity.current = self.inst.components.sanity.max*percent
			self.inst.components.sanity:DoDelta(0) -- Updates HUD
		end
		
		if self.inst.components.combat then
			if self.saveddamage then
				self.inst.components.combat.damagemultiplier = self.saveddamage
			else
				self.inst.components.combat.damagemultiplier = nil
			end
		end
		
		if GetPlayer() ~= self.inst then
			GetPlayer().HUD.controls.revived2:Hide()
		else
			GetPlayer().HUD.controls.revived:Hide()
		end
	
		self.task:Cancel()
        self.task = nil
    end
end

function RevivalSickness:BeginSickness()
	if not self.task then
		
		if self.inst.components.health then
			self.savedhealth = self.inst.components.health.maxhealth
		end
		
		if self.inst.components.hunger then
			self.savedhunger = self.inst.components.hunger.max
		end
		
		if self.inst.components.sanity then
			self.savedsanity = self.inst.components.sanity.max
		end
		
		if self.inst.components.combat and self.inst.components.combat.damagemultiplier then
			self.inst.components.combat.damagemultiplier = self.saveddamage
		end
		
		self:RefreshStats()
		self.task = self.inst:DoPeriodicTask(self.period, function() self:DoUpdate(self.period) end)
	end
	
	self.sicknesstime = self.sicknessmaxtime
end

function RevivalSickness:OnSave()
	if self.sicknesstime > 0 then
		return {
			sicknesstime = self.sicknesstime,
			savedhealth = self.savedhealth,
			savedhunger = self.savedhunger,
			savedsanity = self.savedsanity,
			saveddamage = self.saveddamage,
		}
	end
end

function RevivalSickness:OnLoad(data)
	if data then
		self.sicknesstime = data.sicknesstime
		self.savedhealth = data.savedhealth
		self.savedhunger = data.savedhunger
		self.savedsanity = data.savedsanity
		self.saveddamage = data.saveddamage
		self:RefreshStats()
		if not self.task and self.sicknesstime > 0 then
			self.task = self.inst:DoPeriodicTask(self.period, function() self:DoUpdate(self.period) end)
		end
	end
end

function RevivalSickness:IsSick()
    if self.sicknesstime > 0 then
		return true
	end
end

return RevivalSickness
