require "screen"
require "button"
require "animbutton"
require "image"
require "uianim"

local MAX_HUD_SCALE = 1.25

require "screens/consolescreen"

require("widgets/statusdisplays")
require("widgets/inventorybar")
require("widgets/hoverer")
require("widgets/crafting")
require("widgets/mapwidget")
require("widgets/containerwidget")

local easing = require("easing")

local Sim = TheSim
local gettime = GetTime

local debugstr = {}
local MAX_CONSOLE_LINES = 15
local consolelog = function(...)

    local str = ""
    for i,v in ipairs({...}) do
        str = str..tostring(v)
    end

    table.insert(debugstr, str)

    while #debugstr > MAX_CONSOLE_LINES do
        table.remove(debugstr,1)
    end
end

addprintlogger(consolelog)

-- MOD
local CraftingSlot = 0
local CurrentSlot = 0
local SelectedSlot = 0
local CurrentEquipSlot = 0
local SelectedEquipSlot = 0
local CurrentContainer = nil
local ContainersFound = {}
local EquipSlots = {EQUIPSLOTS.HANDS, EQUIPSLOTS.BODY, EQUIPSLOTS.HEAD}
local modifier1key = KEY_KP_ENTER
local modifier2key = KEY_KP_PLUS
-- /MOD


local Controls = Class(Widget, function(self, owner)
    Widget._ctor(self, "Controls")
    self.owner = owner
	
	-- MOD
	self.player2 = GetPlayer()
    -- /MOD
	
    --overlays
    self.clouds = self:AddChild(UIAnim())
    self.clouds:SetClickable(false)
    self.clouds:SetHAnchor(ANCHOR_MIDDLE)
    self.clouds:SetVAnchor(ANCHOR_MIDDLE)
    self.clouds:GetAnimState():SetBank("clouds_ol")
    self.clouds:GetAnimState():SetBuild("clouds_ol")
    self.clouds:GetAnimState():PlayAnimation("idle", true)
    self.clouds:GetAnimState():SetMultColour(1,1,1,0)
    self.clouds:Hide()

    self.iceover = self:AddChild(IceOver(owner))
    self.iceover:Hide()
    self.fireover = self:AddChild(FireOver(owner))
    self.fireover:Hide()


	self:MakeScalingNodes()

	self.minimap = self:AddChild(MapWidget(self.owner))
	self.minimap:Hide()

	--these are visible above the map
    self.bottomright_root = self:AddChild(Widget(""))
    self.bottomright_root:SetScaleMode(SCALEMODE_PROPORTIONAL)
    self.bottomright_root:SetHAnchor(ANCHOR_RIGHT)
    self.bottomright_root:SetVAnchor(ANCHOR_BOTTOM)
    self.bottomright_root:SetMaxPropUpscale(MAX_HUD_SCALE)
	self.bottomright_root = self.bottomright_root:AddChild(Widget(""))
	
	
	------ CONSOLE -----------	
	
	self.consoletext = self.bottom_root:AddChild(Text(BODYTEXTFONT, 20, "CONSOLE TEXT"))
	self.consoletext:SetVAlign(ANCHOR_BOTTOM)
	self.consoletext:SetHAlign(ANCHOR_LEFT)
	self.consoletext:SetRegionSize(850, 700)
	self.consoletext:SetPosition(0,500,0)
	self.consoletext:Hide()
    -----------------


    
    self.saving = self:AddChild(SavingIndicator(self.owner))
    self.saving:SetHAnchor(ANCHOR_MIDDLE)
    self.saving:SetVAnchor(ANCHOR_TOP)
    self.saving:SetPosition(Vector3(200,0,0))
    
    self.inv = self.bottom_root:AddChild(Inv(self.owner))
    
    self.crafttabs = self.left_root:AddChild(CraftTabs(self.owner))
    local scale = .75
    self.crafttabs:SetScale(scale,scale,scale)
    


	self.sidepanel = self.topright_root:AddChild(Widget("sidepanel"))
	self.sidepanel:SetScale(1,1,1)
	self.sidepanel:SetPosition(-80, -60, 0)

	-- MOD
	self.toppanel = self.top_root:AddChild(Widget("toppanel"))
	self.toppanel:SetScale(1,1,1)
	self.toppanel:SetPosition(0, -60, 0)
	-- /MOD
	
    --self.status = self.sidepanel:AddChild(Status(self.owner)) -- MOD
	--self.status:SetPosition(0,-110,0) -- MOD
    --self.status:SetScale(.9,.9,.9)
	
	self.status = self.toppanel:AddChild(Status(self.owner)) -- MOD
    self.status:SetPosition(-235,0,0) -- MOD
	
    --self.clock = self.sidepanel:AddChild(UIClock(self.owner)) -- MOD
    self.clock = self.toppanel:AddChild(UIClock(self.owner)) -- MOD

    if GetWorld() and GetWorld():IsCave() then
    	self.clock:Hide()
    	--self.status:SetPosition(-10,-20,0) -- MOD
    end
    

	self.containers = {}

    local MAPSCALE = .5
	
	-- MOD
	--self.maproot = self.bottomright_root:AddChild(Widget("mapstuff")) 
	--self.maproot:SetPosition(-60,70,0) 
	self.maproot = self.bottom_root:AddChild(Widget("mapstuff")) 
	self.maproot:SetPosition(0,70,0) 
	-- /MOD
	self.minimapBtn = self.maproot:AddChild(Button())

	local btn = self.minimapBtn
    btn:SetScale(MAPSCALE,MAPSCALE,MAPSCALE)
	btn:SetOnClick( function() self:ToggleMap() end )
	btn:SetMouseOver( function()  btn:ScaleTo(MAPSCALE*1.1,MAPSCALE*1.1,MAPSCALE*1.1) end )
	btn:SetMouseOut( function() btn:ScaleTo(MAPSCALE,MAPSCALE,MAPSCALE) end )
	
	btn:SetTooltip(STRINGS.UI.HUD.MAP)
	
	local hud_atlas = resolvefilepath("images/hud.xml")
	btn:SetImage(hud_atlas, "map_button.tex")


	self.pauseBtn = self.maproot:AddChild(Button())
	
    self.pauseBtn:SetOnClick(
		function()
			if not IsHUDPaused() then
				TheFrontEnd:PushScreen(PauseScreen())
			end
		end )
	
	self.pauseBtn:SetMouseOver(
		function()
			self.pauseBtn:ScaleTo( .4, .4, .4 )
		end )
		
	self.pauseBtn:SetMouseOut(
		function()
			self.pauseBtn:ScaleTo( .33, .33, .33 )
		end )
		
	self.pauseBtn:SetTooltip(STRINGS.UI.HUD.PAUSE)
		
	self.pauseBtn:SetImage( hud_atlas, "pause.tex" )	
	self.pauseBtn:SetScale(.33,.33,.33)
	self.pauseBtn:SetPosition( Point( 0,-50,0 ) )
        
    if true or not IsGamePurchased() then
		self.demotimer = self.top_root:AddChild(DemoTimer(self.owner))
		self.demotimer:SetPosition(320, -25, 0)
	end
	

	self.alphawarning = self.top_root:AddChild(Text(TITLEFONT, 40))
	self.alphawarning:SetRegionSize(400, 50)
	self.alphawarning:SetPosition(0, -28, 0) 
	self.alphawarning:SetString(STRINGS.UI.HUD.CAVEWARNING)
	self.alphawarning:Hide()
	if GetWorld():IsCave() then
		self.alphawarning:Show()
	end


	self.rotleft = self.maproot:AddChild(Button())
	self.rotleft:SetImage( hud_atlas, "turnarrow_icon.tex" )
	--self.rotleft:SetMouseOverImage( "images/hud.xml", "turnarrow_icon_over.tex" )
	
	self.rotleft:SetMouseOver( function() self.rotleft:SetScale( -.8, .8, .8 ) end )
	self.rotleft:SetMouseOut( function() self.rotleft:SetScale( -.7, .7, .7 ) end )	
	
    self.rotleft:SetPosition(-40,-40,0)
    self.rotleft:SetOnClick(function() GetPlayer().components.playercontroller:RotLeft() end)
    self.rotleft:SetScale(-.7,.7,.7)
    self.rotleft:SetTooltip(STRINGS.UI.HUD.ROTLEFT)

	self.rotright = self.maproot:AddChild(Button())
	self.rotright:SetImage( hud_atlas, "turnarrow_icon.tex" )
	--self.rotright:SetMouseOverImage( "images/hud.xml", turnarrow_icon_over.tex" )
	
	self.rotright:SetMouseOver( function() self.rotright:SetScale( .8, .8, .8 ) end )
	self.rotright:SetMouseOut( function() self.rotright:SetScale( .7, .7, .7 ) end )	
	
    self.rotright:SetPosition(40,-40,0)
    self.rotright:SetOnClick(function() GetPlayer().components.playercontroller:RotRight() end)
    self.rotright:SetScale(.7,.7,.7)
	self.rotright:SetTooltip(STRINGS.UI.HUD.ROTRIGHT)

	self.containerroot = self:AddChild(Widget(""))
    self.containerroot:SetHAnchor(ANCHOR_MIDDLE)
    self.containerroot:SetVAnchor(ANCHOR_MIDDLE)
	self.containerroot:SetScaleMode(SCALEMODE_PROPORTIONAL)
	self.containerroot:SetMaxPropUpscale(MAX_HUD_SCALE)
	self.containerroot = self.containerroot:AddChild(Widget(""))
	
	self.containerroot_side = self:AddChild(Widget(""))
    self.containerroot_side:SetHAnchor(ANCHOR_RIGHT)
    self.containerroot_side:SetVAnchor(ANCHOR_MIDDLE)
	self.containerroot_side:SetScaleMode(SCALEMODE_PROPORTIONAL)
	self.containerroot_side:SetMaxPropUpscale(MAX_HUD_SCALE)
	self.containerroot_side = self.containerroot_side:AddChild(Widget(""))
	
    self.hover = self:AddChild(HoverText(self.owner))
    
    self.mousefollow = self:AddChild(Widget("follower"))
    self.mousefollow:FollowMouse(true)
    self.mousefollow:SetScaleMode(SCALEMODE_PROPORTIONAL)
    self.hover:SetScaleMode(SCALEMODE_PROPORTIONAL)

    self:SetHUDSize(Profile:GetHUDSize())
end)


function Controls:MakeScalingNodes()

	--these are auto-scaling root nodes
	self.top_root = self:AddChild(Widget("top"))
    self.top_root:SetScaleMode(SCALEMODE_PROPORTIONAL)
    self.top_root:SetHAnchor(ANCHOR_MIDDLE)
    self.top_root:SetVAnchor(ANCHOR_TOP)
    self.top_root:SetMaxPropUpscale(MAX_HUD_SCALE)

    self.bottom_root = self:AddChild(Widget("bottom"))
    self.bottom_root:SetScaleMode(SCALEMODE_PROPORTIONAL)
    self.bottom_root:SetHAnchor(ANCHOR_MIDDLE)
    self.bottom_root:SetVAnchor(ANCHOR_BOTTOM)
    self.bottom_root:SetMaxPropUpscale(MAX_HUD_SCALE)

    self.topright_root = self:AddChild(Widget("side"))
    self.topright_root:SetScaleMode(SCALEMODE_PROPORTIONAL)
    self.topright_root:SetHAnchor(ANCHOR_RIGHT)
    self.topright_root:SetVAnchor(ANCHOR_TOP)
    self.topright_root:SetMaxPropUpscale(MAX_HUD_SCALE)

    
	self.left_root = self:AddChild(Widget("side"))
    self.left_root:SetScaleMode(SCALEMODE_PROPORTIONAL)
    self.left_root:SetHAnchor(ANCHOR_LEFT)
    self.left_root:SetVAnchor(ANCHOR_MIDDLE)
    self.left_root:SetMaxPropUpscale(MAX_HUD_SCALE)    
	
	--these are for introducing user-configurable hud scale
	self.topright_root = self.topright_root:AddChild(Widget(""))
	self.bottom_root = self.bottom_root:AddChild(Widget(""))
	self.top_root = self.top_root:AddChild(Widget(""))
	self.left_root = self.left_root:AddChild(Widget(""))
	--

end

function Controls:SetHUDSize( size )
	local min_scale = .75
	local max_scale = 1.1
	
	--testing high res displays
	local w,h = TheSim:GetScreenSize()
	
	local res_scale_x = math.max(1, w / 1920)
	local res_scale_y = math.max(1, h / 1200)
	local res_scale = math.min(res_scale_x, res_scale_y)	
	
	local scale = easing.linear(size, min_scale, max_scale-min_scale, 10) * res_scale
	self.topright_root:SetScale(scale,scale,scale)
	self.bottom_root:SetScale(scale,scale,scale)
	self.top_root:SetScale(scale,scale,scale)
	self.bottomright_root:SetScale(scale,scale,scale)
	self.left_root:SetScale(scale,scale,scale)
	self.containerroot:SetScale(scale,scale,scale)
	self.containerroot_side:SetScale(scale,scale,scale)
	self.hover:SetScale(scale,scale,scale)
	
	self.mousefollow:SetScale(scale,scale,scale)
end


-- MOD
function Controls:HLEquipSlot(slot)
	if CurrentEquipSlot ~= 0 and CurrentEquipSlot ~= SelectedEquipSlot then
		self.inv2.equip[EquipSlots[slot]]:ScaleTo(1.3, 1, 99999)
		if self:GetCurrentItem() then
			self.inv2.equip[EquipSlots[slot]].tile:OnMouseOver()
		end
	end
end

function Controls:RemoveHLEquipSlot(slot)
	if CurrentEquipSlot ~= 0 and CurrentEquipSlot ~= SelectedEquipSlot then
		self.inv2.equip[EquipSlots[slot]]:ScaleTo(1.3, 1, 0.25)				
		if self:GetCurrentItem() then
			self.inv2.equip[EquipSlots[slot]].tile:OnMouseOut()
		end
	end
end

function Controls:HLInvSlot(slot)
	if CurrentSlot ~= 0 and CurrentSlot ~= SelectedSlot and CurrentContainer then
		self.containers[CurrentContainer.name].inv[slot]:ScaleTo(1.3, 1, 99999)
		if self:GetCurrentItem() then
			self.containers[CurrentContainer.name].inv[slot].tile:OnMouseOver()
		end
	end
	if CurrentSlot ~= 0 and CurrentSlot ~= SelectedSlot and not CurrentContainer then
		self.inv2.inv[slot]:ScaleTo(1.3, 1, 99999)
		if self:GetCurrentItem() then
			self.inv2.inv[slot].tile:OnMouseOver()
		end
	end
end

function Controls:RemoveHLInvSlot(slot)
	if slot ~= 0 and (CurrentSlot ~= 0 and CurrentSlot ~= SelectedSlot) and CurrentContainer then
		self.containers[CurrentContainer.name].inv[slot]:ScaleTo(1.3, 1, 0.25)	
		if self:GetCurrentItem() then
			self.containers[CurrentContainer.name].inv[slot].tile:OnMouseOut()
		end
	end
	if slot ~= 0 and (CurrentSlot ~= 0 and CurrentSlot ~= SelectedSlot) and not CurrentContainer then
		self.inv2.inv[slot]:ScaleTo(1.3, 1, 0.25)				
		if self:GetCurrentItem() then
			self.inv2.inv[slot].tile:OnMouseOut()
		end
	end
end

function Controls:GetCurrentItem()
	if not CurrentContainer then
		return (self.player2.components.inventory:GetItemInSlot(CurrentSlot) or self.player2.components.inventory:GetEquippedItem(EquipSlots[CurrentEquipSlot]))
	else
		return CurrentContainer.name.components.container:GetItemInSlot(CurrentSlot)
	end
end

function Controls:GetSelectedItem()
	if not CurrentContainer then
		return (self.player2.components.inventory:GetItemInSlot(SelectedSlot) or self.player2.components.inventory:GetEquippedItem(EquipSlots[SelectedEquipSlot]))
	else
		return CurrentContainer.name.components.container:GetItemInSlot(SelectedSlot)
	end
end

function Controls:GetNumInvSlots()
	return self.player2.components.inventory:GetNumSlots()
end

function Controls:GetNumEquipSlots()
	return self.owner.components.inventory:GetNumEquipSlots()
end

function Controls:FindNextContainer(n)
	local CurrentContainerID = 0
	local CurrentContainerFound = false
	
	for k,v in pairs(ContainersFound) do
		if v.name == CurrentContainer.name then
			CurrentContainerID = k
			CurrentContainerFound = true
		end	
	end
	
	local NumContainersFound = #ContainersFound
	local NextContainerID = CurrentContainerID + n
	
	if NumContainersFound ~= 0 and (NextContainerID < 1 or NextContainerID > NumContainersFound) then
		CurrentContainer = nil
		CurrentSlot = 1
		
	elseif NumContainersFound ~= 0 then
		CurrentContainer = {}
		CurrentContainer.name = ContainersFound[NextContainerID].name
		CurrentContainer.size = ContainersFound[NextContainerID].size
	
		if n < 0 then 
			CurrentSlot = 1
			SelectedSlot = 0
		else
			CurrentSlot = CurrentContainer.size
			SelectedSlot = 0
		end
		
	elseif NumContainersFound == 0 then
		if n < 0 then
			CurrentSlot = CurrentSlot - self:GetNumInvSlots()
			CurrentContainer = nil
		else
			CurrentSlot = self:GetNumInvSlots() + CurrentSlot
			CurrentContainer = nil
		end
	
	end
end

function Controls:InventoryItemSwitching()
	local Item_current = self:GetCurrentItem()
	local Item_selected = self:GetSelectedItem()
	
	if CurrentEquipSlot == 0 and SelectedEquipSlot == 0 then
		local SpecialActions = nil
		if self:GetCurrentItem() and self.player2.components.inventory:GetActiveItem() then
			SpecialActions = self.player2.components.playeractionpicker:GetUseItemActions(self:GetCurrentItem(), self.player2.components.inventory:GetActiveItem(), true)
		end
			
		--[[if SelectedSlot ~= 0 and SpecialActions then	
			self.player2.components.locomotor:PushAction(SpecialActions[1], true)]]
	
		if not CurrentContainer then
			self.player2.components.inventory.itemslots[CurrentSlot] = Item_selected
			self.player2.components.inventory.itemslots[SelectedSlot] = Item_current
			
		else
			CurrentContainer.name.components.container:RemoveItem(Item_current, true)
			CurrentContainer.name.components.container:RemoveItem(Item_selected, true)
			CurrentContainer.name.components.container:GiveItem(Item_current, SelectedSlot)
			CurrentContainer.name.components.container:GiveItem(Item_selected, CurrentSlot)			
		end
	end
	
	self.inv2:Refresh()
end

function Controls:ClearActiveItemState()	
	if SelectedSlot ~= 0 and CurrentSlot ~= SelectedSlot then
		if not CurrentContainer then
			self.inv2.inv[SelectedSlot]:ScaleTo(1.3, 1, 0.25)
		else
			self.containers[CurrentContainer.name].inv[SelectedSlot]:ScaleTo(1.3, 1, 0.25)
		end
		
		self:InventoryItemSwitching()		
	end
	
	SelectedSlot = 0		
	self.player2.components.inventory:SetActiveItem(nil)	
end
-- /MOD


function Controls:OnKeyUp( key )

	-- MOD
	local CurrentItem = self:GetCurrentItem()
	local SelectedItem = self:GetSelectedItem()
	local NumInvSlots = self:GetNumInvSlots()
	local NumEquipSlots = self:GetNumEquipSlots()
	local CurrentContainerFound = false
	
	-- Create a table of open containers
	ContainersFound = {}
	for k,v in pairs(self.containers) do
		if v and v.isopen then
			table.insert(ContainersFound, {id = k, name = v.container, size = v.container.components.container:GetNumSlots()})
		end
	end
	
	-- Sort these containers by size
	table.sort(ContainersFound, function(a,b) return a.size < b.size end)
	
	-- Return  to the second players inventory if the container the second player was browsing closes 
	for k,v in pairs(ContainersFound) do
		if v.name and v.name == CurrentContainer.name then
			CurrentContainerFound = true
		end
	end
	
	

	if CurrentContainer and not CurrentContainerFound then
		CurrentContainer = nil
		CurrentSlot = 1
	end
	-- /MOD
	
	if self.owner and self.owner.components.playercontroller and self.owner.components.playercontroller.enabled then
		if key >= KEY_0 and key <= KEY_9 then
			local num = 1
			if key == KEY_0 then
				num = 10
			else
				num = key - KEY_1 + 1
			end
			self.inv:UseSlot(num)
		elseif key == KEY_MINUS then
			self.inv:UseSlot(11)
		elseif key == KEY_EQUALS then
			self.inv:UseSlot(12)
	    elseif key == KEY_TAB then
		    if not GetPlayer():HasTag("beaver") then
				print ("toggle map!")
				self:ToggleMap()
			end
	    elseif key == KEY_TILDE and PLATFORM ~= "NACL" and TheSim:GetSetting("MISC", "ENABLECONSOLE") == "true" then
	    	TheFrontEnd:PushScreen(ConsoleScreen())
	    	self.consoletext:Show()
	    elseif key == KEY_L and TheInput:IsKeyDown(KEY_CTRL) then
	    	if self.consoletext.shown then
		    	self.consoletext:Hide()
		    else
		    	self.consoletext:Show()
		    end
	    end
	end
	
	-- MOD
	if self.player2 and self.player2.components.playercontroller and self.player2.components.playercontroller.enabled then
		-- Select key	
		if key == modifier1key then
			-- If an item is in a slot, and no item is selected, select it
			if SelectedSlot == 0 and CurrentSlot ~= 0 and self:GetCurrentItem() then								
				SelectedSlot = CurrentSlot
				self.player2.components.inventory:SetActiveItem(self:GetCurrentItem())
				if not CurrentContainer then
					self.inv2.inv[CurrentSlot].tile:OnMouseOut()
				else
					self.containers[CurrentContainer.name].inv[CurrentSlot].tile:OnMouseOut()
				end
			-- If an item is selected, and the slot is highlighted, deselect it without shrinking it
			elseif self:GetSelectedItem() and ((SelectedSlot ~= 0 and SelectedSlot == CurrentSlot) or (SelectedEquipSlot ~= 0 and SelectedEquipSlot == CurrentEquipSlot)) then
				SelectedSlot = 0
				self.player2.components.inventory:SetActiveItem(nil)
			
			-- If an item is selected deselect it
			elseif self:GetSelectedItem() then
				self:ClearActiveItemState()
			end		
		
		elseif key == KEY_KP_1 then
			
		elseif key == KEY_KP_2 then
			-- Deselect the previous slot
			self:RemoveHLInvSlot(CurrentSlot)
			self:RemoveHLEquipSlot(CurrentEquipSlot)
			
			-- Move to new slot
			if CurrentContainer and CurrentContainer.size == 9 then
				CurrentSlot = CurrentSlot + 3
				if CurrentSlot > CurrentContainer.size then
					self:FindNextContainer(-1)
				end
				CurrentEquipSlot = 0
				
			elseif CurrentContainer then
				CurrentSlot = CurrentSlot + 1
				if CurrentSlot > CurrentContainer.size then
					self:FindNextContainer(-1)
				end
				CurrentEquipSlot = 0
				
			elseif CurrentSlot ~= 0 then
				CurrentSlot = CurrentSlot + 5
				if CurrentSlot > self:GetNumInvSlots() then
					self:FindNextContainer(-1)
				end
				CurrentEquipSlot = 0
				
			elseif CurrentEquipSlot ~= 0 then
				CurrentEquipSlot = CurrentEquipSlot + 1
				if CurrentEquipSlot > self:GetNumEquipSlots() then
					CurrentEquipSlot =  CurrentEquipSlot - self:GetNumEquipSlots()
				end
				CurrentSlot = 0		
				
			else
				CurrentContainer = nil
				CurrentSlot = 1
				CurrentEquipSlot = 0
			end
				
			-- Highlight new slot
			self:HLInvSlot(CurrentSlot)
			self:HLEquipSlot(CurrentEquipSlot)
			
		--elseif key == KEY_KP_3 then
		
		-- Menu scroll (left)
		elseif key == KEY_KP_4 then
			
			-- Deselect the previous slot
			self:RemoveHLInvSlot(CurrentSlot)
			self:RemoveHLEquipSlot(CurrentEquipSlot)
			
			-- Move to new slot
			local n = 0
			if not CurrentContainer then
				n = (CurrentSlot - 1)/5 + 1
			elseif CurrentContainer.size == 9 then
				n = (CurrentSlot - 1)/3 + 1
			end
			
			if CurrentSlot ~= 0 then
				if math.floor(n) == n and not CurrentContainer then
					CurrentSlot = 0
					CurrentEquipSlot = n
					
				elseif math.floor(n) == n and CurrentContainer then
					-- Null
				
				else
					CurrentSlot = CurrentSlot - 1
					CurrentEquipSlot = 0
				end
							
			elseif CurrentEquipSlot ~= 0 then			
				CurrentSlot = 5*CurrentEquipSlot
				CurrentEquipSlot = 0				
			else		
				CurrentSlot = 1
				CurrentEquipSlot = 0
			end
				
			-- Highlight new slot
			self:HLInvSlot(CurrentSlot)
			self:HLEquipSlot(CurrentEquipSlot)
						
		elseif key == KEY_KP_5 then
			local rad = 3
			
			-- Find nearby traders
			local entity = FindEntity(self.player2, rad, function(guy) 
				return guy and not guy.components.trader
			end)
			
			-- Find nearby crops
			if entity and entity.components.grower and entity.components.grower.crops then
				for k,v in pairs(entity.components.grower.crops) do				
					entity = k
				end
			end

			local NormActions = self.player2.components.playeractionpicker:GetInventoryActions(self:GetCurrentItem())
			local SpecialActions = nil
			
			-- Use item in inventory
			if self:GetCurrentItem() and self.player2.components.inventory:GetActiveItem() then
				SpecialActions = self.player2.components.playeractionpicker:GetUseItemActions(self:GetCurrentItem(), self.player2.components.inventory:GetActiveItem(), true)
			end
			
			-- Look at inventory item
			if TheInput:IsKeyDown(modifier2key) then
				local action = BufferedAction(self.player2, self:GetCurrentItem(), ACTIONS.LOOKAT)
				self.player2.components.locomotor:PushAction(action, true)
			
			-- Use item on world entity
			elseif entity and self:GetCurrentItem() and self:GetSelectedItem() == self:GetCurrentItem() and TheInput:IsKeyDown(modifier1key) then
				local specialaction = self.player2.components.playeractionpicker:GetUseItemActions(entity, self:GetCurrentItem(), true)
				if specialaction then			
					self.player2.components.locomotor:PushAction(specialaction[1], true)
				end
				
			-- Deploy item
			elseif self:GetCurrentItem() and self:GetCurrentItem().components.deployable and TheInput:IsKeyDown(modifier1key) then
				local action = BufferedAction(self.player2, nil, ACTIONS.DEPLOY, self:GetCurrentItem(), Vector3(self.player2.Transform:GetWorldPosition()))
				self.player2.components.locomotor:PushAction(action, true)	
				
			-- Other special actions	
			elseif self:GetCurrentItem() and self:GetSelectedItem() and TheInput:IsKeyDown(modifier1key) then
				local specialaction = self.player2.components.playeractionpicker:GetUseItemActions(self:GetCurrentItem(), self:GetSelectedItem(), true)
				if specialaction then			
					self.player2.components.locomotor:PushAction(specialaction[1], true)
				end
			
			else	
				if SelectedSlot == 0 and NormActions then							
					self.player2.components.locomotor:PushAction(NormActions[1], true)
					
				elseif SelectedSlot ~= 0 and SpecialActions then	
					self.player2.components.locomotor:PushAction(SpecialActions[1], true)
					if self:GetSelectedItem() and self:GetSelectedItem().components.stackable and self:GetSelectedItem().components.stackable.stacksize < 2 then
						self.inv2.inv[SelectedSlot]:ScaleTo(1.3, 1, 0.25)				
						self.inv2.inv[SelectedSlot].tile:OnMouseOut()

						SelectedSlot = 0
						self.player2.components.inventory:SetActiveItem(nil)
						self.inv2:Refresh()
						if CurrentSlot ~= 0 then
							self.inv2.inv[CurrentSlot].tile:OnMouseOver()
						else
							self.inv2.equip[EquipSlots[CurrentEquipSlot]].tile:OnMouseOver()
						end
					end
				end
			end
			
		elseif key == KEY_KP_6 then
			
			-- Deselect the previous slot
			self:RemoveHLInvSlot(CurrentSlot)
			self:RemoveHLEquipSlot(CurrentEquipSlot)
			
			-- Move to new slot
			local n = 0
			if not CurrentContainer then
				n = CurrentSlot/5
			elseif CurrentContainer.size == 9 then
				n = CurrentSlot/3
			end
			
			if CurrentSlot ~= 0 then
				if math.floor(n) == n and not CurrentContainer then
					CurrentSlot = 0
					CurrentEquipSlot = n
				elseif math.floor(n) == n and CurrentContainer then
					-- Null
				else
					CurrentSlot = CurrentSlot + 1
					CurrentEquipSlot = 0
				end
							
			elseif CurrentEquipSlot ~= 0 then			
				CurrentSlot = (CurrentEquipSlot-1)*5+1
				CurrentEquipSlot = 0				
			
			else		
				CurrentSlot = 1
				CurrentEquipSlot = 0
			end
				
			-- Highlight new slot
			self:HLInvSlot(CurrentSlot)
			self:HLEquipSlot(CurrentEquipSlot)
			
		elseif key == KEY_KP_7 then
			local action_done = nil
			local rad = 2.5
			
			-- Find a nearby trading partner
			local tradingpartner = FindEntity(self.player2, rad, function(dude)
					return (dude) and dude == self.player2.otherpartymember or dude.components.trader
			end)
			
			-- Trade the item
			if tradingpartner and self:GetCurrentItem() and not action_done and not CurrentContainer then	
				local action = BufferedAction(self.player2, tradingpartner, ACTIONS.GIVE, self:GetCurrentItem())
				self.player2.components.locomotor:PushAction(action, true)
				action_done = true
			end
			
			-- Find nearby entities
			local entity = FindEntity(self.player2, rad, function(guy) 
				return guy		
			end)
			
			-- Unequip an item
			if CurrentEquipSlot ~= 0 and self:GetCurrentItem() and not action_done then								
				local action = BufferedAction(self.player2, nil, ACTIONS.UNEQUIP, self:GetCurrentItem())
				self.player2.components.locomotor:PushAction(action, true)
				action_done = true
			end	
			
			-- Combine the current item with the closest entity
			--[[if entity and self:GetCurrentItem() and not action_done and not CurrentContainer then
				local specialaction = self.player2.components.playeractionpicker:GetUseItemActions(entity, self:GetCurrentItem(), true)
				if specialaction then			
					self.player2.components.locomotor:PushAction(specialaction[1], true)
					action_done = true
					if entity.components.container then
						--entity.components.container:Open()
					end
				end]]
			
			-- Pickup an item inside a chest
			if entity and entity.components.container and self:GetCurrentItem() and not action_done and CurrentContainer then
				local action = BufferedAction(self.player2, self:GetCurrentItem(), ACTIONS.PICKUP)
				self.player2.components.locomotor:PushAction(action, true) 
				action_done = true
			end	
			
			-- Drop item into a chest
			if entity and entity.components.container and self:GetCurrentItem() and not action_done and ContainersFound then
				local action = self.player2.components.playeractionpicker:GetUseItemActions(entity, self:GetCurrentItem(), true)
				if action then			
					self.player2.components.locomotor:PushAction(action[1], true)
				end
				
				action_done = true
			end	
			
			-- Drop a single item on the ground
			if CurrentSlot ~= 0 and TheInput:IsKeyDown(modifier1key) and not action_done then
				self.player2.components.inventory:DropItem(self:GetCurrentItem())
			
			-- Drop a stack of items on the ground
			elseif CurrentSlot ~= 0 and not action_done then
				self.player2.components.inventory:DropItem(self:GetCurrentItem(), true)
			end
			
			self:ClearActiveItemState()
			
		elseif key == KEY_KP_8 then
			-- Deselect the previous slot
			self:RemoveHLInvSlot(CurrentSlot)
			self:RemoveHLEquipSlot(CurrentEquipSlot)
			
			-- Move to new slot
			if CurrentContainer and CurrentContainer.size == 9 then
				CurrentSlot = CurrentSlot - 3
				if CurrentSlot < 1 then
					self:FindNextContainer(1)
				end
				CurrentEquipSlot = 0
				
			elseif CurrentContainer then
				CurrentSlot = CurrentSlot - 1
				if CurrentSlot < 1 then
					self:FindNextContainer(1)
				end
				CurrentEquipSlot = 0
				
			elseif CurrentSlot ~= 0 then
				CurrentSlot = CurrentSlot - 5
				if CurrentSlot < 1 then
					self:FindNextContainer(1)
				end
				CurrentEquipSlot = 0
				
			elseif CurrentEquipSlot ~= 0 then
				CurrentEquipSlot = CurrentEquipSlot - 1
				if CurrentEquipSlot < 1 then
					CurrentEquipSlot =  CurrentEquipSlot + self:GetNumEquipSlots()
				end
				CurrentSlot = 0		
				
			else
				CurrentContainer = nil
				CurrentSlot = 1
				CurrentEquipSlot = 0
			end
				
			-- Highlight new slot
			self:HLInvSlot(CurrentSlot)
			self:HLEquipSlot(CurrentEquipSlot)
		
		-- Deploy current item
		elseif key == KEY_KP_9 then
		
			--[[local tradingpartner = FindEntity(self.player2, 2.5, function(dude)
					return (dude) and dude == self.owner
			end)
			
			if not tradingpartner then
				tradingpartner = FindEntity(self.player2, 8, function(dude)
					return (dude.components.trader) and not dude:HasTag("player")
				end)
			end
			
			if tradingpartner then	
				local action = BufferedAction(self.player2, tradingpartner, ACTIONS.GIVE, self:GetCurrentItem())
				self.player2.components.locomotor:PushAction(action, true)
			end]]
	    end
	end
	-- /MOD
end

function Controls:Update(dt)
    
    local consolestr = ""
    for i,v in ipairs(debugstr) do
    	consolestr = consolestr..v.."\n"
    end
    consolestr = consolestr.."(Press CTRL+L to close this log)"

   	self.consoletext:SetString(consolestr)

    self.status:Update(dt)
    self.hover:Update(dt)
    self.fireover:Update(dt)
    self.crafttabs:Update(dt)
    self.iceover:Update(dt)
	
    -- MOD
	if self.status2 then
		self.status2:Update(dt)
	end
	-- /MOD
	
    for k,v in pairs(self.containers) do
		if not v:Update(dt) then
			self.containers[k] = nil
			v:Kill()
		end
	end
    
    
    if not GetWorld():IsCave() then
	    --this is kind of a weird place to do all of this, but the anim *is* a hud asset...
	    if TheCamera and TheCamera.distance and not TheCamera.dollyzoom then
	        local dist_percent = (TheCamera.distance - TheCamera.mindist) / (TheCamera.maxdist - TheCamera.mindist)
	        local cutoff = .6
	        if dist_percent > cutoff then
	            if not self.clouds_on then
					TheCamera.should_push_down = true
	                self.clouds_on = true
	                self.clouds:Show()
	                self.owner.SoundEmitter:PlaySound("dontstarve/common/clouds", "windsound")
	                TheMixer:PushMix("high")
	            end
	            
	            local p = easing.outCubic( dist_percent-cutoff , 0, 1, 1 - cutoff) 
	            self.clouds:GetAnimState():SetMultColour(1,1,1, p )
	            self.owner.SoundEmitter:SetVolume("windsound",p)
	        else
	            if self.clouds_on then
					TheCamera.should_push_down = false
	                self.clouds_on = false
	                self.clouds:Hide()
	                self.owner.SoundEmitter:KillSound("windsound")
	                TheMixer:PopMix("high")
	            end
	        end
	    end
	end
	
    if self.demotimer then
		if IsGamePurchased() then
			self.demotimer:Kill()
			self.demotimer = nil
		else
			self.demotimer:Update(dt)
		end
	end
end


function Controls:ToggleMap()
    local minimap = self.owner.HUD.minimap
    if minimap then
        minimap.MiniMap:ToggleVisibility()
        if minimap.MiniMap:IsVisible() then
            self.containerroot:Hide()
            self.containerroot_side:Hide()
            self.owner.SoundEmitter:PlaySound("dontstarve/HUD/map_open")
            SetHUDPause(true,"minimap")
			self.minimap:Show()
			self.minimap:Update()
            self.minimapBtn:Show()
            self.pauseBtn:Hide()
            self.hover:Hide()
            self.rotleft:Hide()
            self.rotright:Hide()
            
            if self.inv.hovertile then
				self.inv.hovertile:Hide()
			end
        else
			self.containerroot:Show()
			self.containerroot_side:Show()
            self.owner.SoundEmitter:PlaySound("dontstarve/HUD/map_close")
            SetHUDPause(false)
			self.minimap:Hide()
			self.pauseBtn:Show()
			self.hover:Show()
            if self.inv.hovertile then
				self.inv.hovertile:Show()
			end
            self.rotleft:Show()
            self.rotright:Show()

			
        end
    end
end

PlayerHud = Class(Screen, function(self)
	Screen._ctor(self, "HUD")
    
    
    self.vig = self:AddChild(UIAnim())
    self.vig:GetAnimState():SetBuild("vig")
    self.vig:GetAnimState():SetBank("vig")
    self.vig:GetAnimState():PlayAnimation("basic", true)
    self.vig:SetHAnchor(ANCHOR_LEFT)
    self.vig:SetVAnchor(ANCHOR_TOP)
    self.vig:SetScaleMode(SCALEMODE_FIXEDPROPORTIONAL)
    self.vig:SetClickable(false)
    
    self.bloodover = self:AddChild(BloodOver())
    
    self.minimap = SpawnPrefab( "minimap" )
    self.under_root = self:AddChild(Widget("under_root"))
    self.root = self:AddChild(Widget("root"))
end)

function PlayerHud:OnGainFocus()
	Screen.OnGainFocus(self)
	if self.controls then
		self.controls:SetHUDSize(Profile:GetHUDSize())
	end
end

function PlayerHud:OnDestroy()
	self.minimap:Remove()
	Screen.OnDestroy(self)
end

function PlayerHud:Hide()
	self.shown = false
	self.root:Hide()
end

function PlayerHud:Show()
	self.shown = true
	self.root:Show()
end


function PlayerHud:CloseContainer(container)
    for k,v in pairs(self.controls.containers) do
		if v.container == container then
			v:Close()
		end
    end
end

function PlayerHud:OpenContainer(container, side)

	if container then
		local containerwidget = nil
		if side then
			containerwidget = self.controls.containerroot_side:AddChild(ContainerWidget(self.owner))
		else
			containerwidget = self.controls.containerroot:AddChild(ContainerWidget(self.owner))
		end
		containerwidget:Open(container, self.owner)
	    
		for k,v in pairs(self.controls.containers) do
			if v.container then
				if v.container.prefab == container.prefab then
					v:Close()
				end
			else
				self.controls.containers[k] = nil
			end
		end
	    
		self.controls.containers[container] = containerwidget
	end
	    
end

function PlayerHud:GoSane()
    if self.owner.components.sanity and 
		self.owner.components.sanity.sane and 
		self.owner.otherpartymember and 
		self.owner.otherpartymember.components.sanity and 
		self.owner.otherpartymember.components.sanity.sane then
		
		self.vig:GetAnimState():PlayAnimation("basic", true)
	end
end

function PlayerHud:GoInsane()
    self.vig:GetAnimState():PlayAnimation("insane", true)
end

function PlayerHud:SetMainCharacter(maincharacter)
    if maincharacter then
		maincharacter.HUD = self
		self.owner = maincharacter
		
		self.controls = self.root:AddChild(Controls(self.owner))
		self.controls.inv:Refresh()
		--self.inst.entity:SetParent(self.inst.entity)

		self.inst:ListenForEvent("badaura", function(inst, data) return self.bloodover:Flash() end, self.owner)
		self.inst:ListenForEvent("attacked", function(inst, data) return self.bloodover:Flash() end, self.owner)
		self.inst:ListenForEvent("startstarving", function(inst, data) self.bloodover:TurnOn() end, self.owner)
		self.inst:ListenForEvent("stopstarving", function(inst, data) self.bloodover:TurnOff() end, self.owner)
		self.inst:ListenForEvent("gosane", function(inst, data) self:GoSane() end, self.owner)
		self.inst:ListenForEvent("goinsane", function(inst, data) self:GoInsane() end, self.owner)
		
		if not self.owner.components.sanity:IsSane() then
			self:GoInsane()
		end
		self.controls.crafttabs:UpdateRecipes()
		
		local bp = maincharacter.components.inventory:GetEquippedItem(EQUIPSLOTS.BODY)
		if bp and bp.components.container then
			bp.components.container:Close()
			bp.components.container:Open(maincharacter)
		end
	end
end

function PlayerHud:IsMapShowing()
    return self.minimap and self.minimap.MiniMap:IsVisible()
end

function PlayerHud:OnUpdate(dt)
    if self.owner then
		if self.controls then
			self.controls:Update(dt)
		end

		if self.bloodover then
			self.bloodover:Update(dt)
		end
		
		if Profile and self.vig then
			if RENDER_QUALITY.LOW == Profile:GetRenderQuality() then
				self.vig:Hide()
			else
				self.vig:Show()
			end
		end
	end
end

function PlayerHud:OnKeyUp( key )
	if key == KEY_ESCAPE then
		if not IsHUDPaused() then
			TheFrontEnd:PushScreen(PauseScreen())
		end
	end
end

function PlayerHud:OnKeyDown( key )
	local focused = self:GetActiveFocusWidget()
	if self.controls then
		self.controls:OnKeyUp( key )
	end
end

return PlayerHud

